@@ -1,6 +1,6 @@
 /*
    Egon Animator
-   Copyright (C) 1997  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1997-1999  Ulric Eriksson <ulric@edu.stockholm.se>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,39 +14,31 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+   MA 02111-1307, USA.
  */
 
-/*
- *      cmds.c
- *
- *      This rather bulky module contains all the functions that implement
- *      commands.  It also handles initialization of the interface to those
- *      functions.
- */
+/* ---
+      cmds.c
+
+      This rather bulky module contains all the functions that implement
+      commands.  It also handles initialization of the interface to those
+      functions.
+--- */
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <unistd.h>
-#include <X11/Xatom.h>
-#include <X11/Xlib.h>
-#include <X11/Intrinsic.h>
-#include <X11/Xaw/Label.h>
-#include <X11/xpm.h>
 
-#include "../xcommon/Animator.h"
-#include "egon.h"
-#include "types.h"
-#include "../siod/siod.h"
-#include "../common/fonts.h"
-#include "../xcommon/embed.h"
+#include "../common/common.h"
 #include "../common/cmalloc.h"
-#include "../xcommon/dialogs.h"
-#include "../xcommon/filesel.h"
+#include "../common/plugin.h"
+#include "../siod/siod.h"
 
-extern int input_warp_pointer;	/* from input.c */
+#include "egon.h"
+#include "user_interface.h"
 
 typedef struct {
 	char *name;
@@ -62,9 +54,11 @@ static char *testlist[] = {	"testitem0",
 	"testitem6", "testitem7", "testitem8", "testitem9"
 };
 
+/* ---
+*/
 static LISP listsel_test(void)
 {
-	int i = listsel(topLevel, "Choose an item", testlist, 10);
+	int i = select_from_list("Choose an item", testlist, 10);
 	if (i < 0)
 		printf("User clicked Cancel\n");
 	else
@@ -72,11 +66,15 @@ static LISP listsel_test(void)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP what_cursor_position(void)
 {
 	return NIL;
 }
 
+/* ---
+*/
 static LISP spawn(LISP command)
 {
 	char *argv[20], *p, cmd[1024];
@@ -99,28 +97,26 @@ static LISP spawn(LISP command)
 
 /* Windows and buffers */
 
+/* ---
+*/
 static LISP ask_for_file(void)
 {
 	static char path[1024], name[1024];
 	char fn[1024];
 	char fmt[80];
-	char extra[1024];
 	if (path[0] == '\0')
 		getcwd(path, 1024);
 	name[0] = fn[0] = '\0';
-	sprintf(extra, "Home=%s:Examples=%s/examples/egon",
-		getenv("HOME"), siagdocs);
-	if (!fsel_input(topLevel, path, name, NULL, fmt, extra))
+	if (!select_file(path, name, NULL, fmt))
 		return NIL;
 	sprintf(fn, "%s/%s", path, name);
 	return strcons(strlen(fn), fn);
 }
 
-/*X
-   static void load_buffer()
-
+/* ---
    Load a buffer from file.
-   X */
+*/
+
 static LISP load_buffer(void)
 {
 	static char path[1024], name[1024];
@@ -128,7 +124,6 @@ static LISP load_buffer(void)
 	char fmt[80];
 	buffer *b;
 	static int need_init = 1;
-        char extra[1024];
 
 	if (need_init) {
 		getcwd(path, 1024);
@@ -136,9 +131,7 @@ static LISP load_buffer(void)
 	}
 	strcpy(name, "");
 	fn[0] = '\0';
-        sprintf(extra, "Home=%s:Examples=%s/examples/egon",
-                getenv("HOME"), siagdocs);
-	if (fsel_input(topLevel, path, name, patterns, fmt, extra)) {
+	if (select_file(path, name, loader_patterns, fmt)) {
 		sprintf(fn, "%s/%s", path, name);
 		b = new_buffer(buffer_name(fn), fn);
 		llpr("Loading");
@@ -159,12 +152,14 @@ static LISP load_buffer(void)
 	return NIL;
 }
 
-/* Load using an external program
+/* ---
+Load using an external program
 
    1. Ask for the program to use
    2. Run the program and save output to file in /tmp
    3. Load the file using NULL format (i.e. ask for type)
 */
+
 static LISP load_external(void)
 {
 	static int loaders = 0;
@@ -179,7 +174,7 @@ static LISP load_external(void)
 		char fnl[1024];
 		char *p, *q, b[256];
 
-		sprintf(fnl, "%s/egon/external.load", siaghome);
+		sprintf(fnl, "%s/egon/external.load", datadir);
 		if ((fp = fopen(fnl, "r")) == NULL) {
 			llpr("Can't open header file");
 			return NIL;
@@ -196,7 +191,7 @@ static LISP load_external(void)
 		need_init = 0;
 	}
 	program[0] = param[0] = '\0';
-	i = listsel(topLevel, "External Program:", loadname, loaders);
+	i = select_from_list("External Program:", loadname, loaders);
 
 	if (i >= 0 && ask_for_str("Parameters:", param)) {
 		sprintf(fn, "/tmp/egon%ld", (long)getpid());
@@ -220,6 +215,8 @@ static LISP load_external(void)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP delete_window(void)
 {
 	if (!remove_window(w_list))
@@ -230,6 +227,8 @@ static LISP delete_window(void)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP delete_other_windows(void)
 {
 	while (remove_window(w_list->next));
@@ -237,6 +236,8 @@ static LISP delete_other_windows(void)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP split_window_vertically(void)
 {
 	if (!split_window(w_list))
@@ -246,6 +247,8 @@ static LISP split_window_vertically(void)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP other_window(void)
 {
 	activate_window(w_list->next);
@@ -253,17 +256,16 @@ static LISP other_window(void)
 	return NIL;
 }
 
-/*X
-   static void save_buffer()
-
+/* ---
    Save the buffer in the currently active window to file.
-   X */
+*/
+
 static LISP save_buffer(void)
 {
 	llpr("Saving");
 
 	if (savematrix(w_list->buf->path, w_list->buf, NULL))
-		errorbox(topLevel, "Couldn't save");
+		error_box("Couldn't save");
 	else {
 		w_list->buf->change = FALSE;
 		pr_scr_flag = TRUE;
@@ -271,15 +273,13 @@ static LISP save_buffer(void)
 	return NIL;
 }
 
-/*X
-   static void save_buffer_as()
-
+/* ---
    Save the buffer in the currently active window to a named file.
-   X */
+*/
+
 static LISP save_buffer_as(void)
 {
 	static char path[1024], name[1024];
-	char extra[1024];
 	char fn[1024];
 	char fmt[80];
 	char *p;
@@ -293,13 +293,11 @@ static LISP save_buffer_as(void)
 	if (p) strcpy(name, p+1);
 	else strcpy(name, w_list->buf->path);
 	fn[0] = '\0';
-        sprintf(extra, "Home=%s:Examples=%s/examples/pw",
-                getenv("HOME"), siagdocs);
-	if (fsel_input(topLevel, path, name, patterns, fmt, extra)) {
+	if (select_file(path, name, saver_patterns, fmt)) {
 		sprintf(fn, "%s/%s", path, name);
 		llpr("Saving");
 		if (savematrix(fn, w_list->buf, fmt))
-			errorbox(topLevel, "Couldn't save");
+			error_box("Couldn't save");
 		else {
 			w_list->buf->change = FALSE;
 			strcpy(w_list->buf->path, fn);
@@ -309,12 +307,14 @@ static LISP save_buffer_as(void)
 	return NIL;
 }
 
-/* Save using an external program
+/* ---
+Save using an external program
 
    1. Ask for the program to use
    2. Save to a file in /tmp using NULL format (i.e. ask for type)
    3. Run the program and read the file as input
 */
+
 static LISP save_external(void)
 {
 	static int savers = 0;
@@ -328,7 +328,7 @@ static LISP save_external(void)
 		char fnl[1024];
 		char *p, *q, b[256];
 
-		sprintf(fnl, "%s/egon/external.save", siaghome);
+		sprintf(fnl, "%s/egon/external.save", datadir);
 		if ((fp = fopen(fnl, "r")) == NULL) {
 			llpr("Can't open saver file");
 			return NIL;
@@ -346,14 +346,14 @@ static LISP save_external(void)
 	}
 
 	program[0] = param[0] = '\0';
-	i = listsel(topLevel, "External Program:", savename, savers);
+	i = select_from_list("External Program:", savename, savers);
 
 	if (i >= 0 && ask_for_str("Parameters:", param)) {
 		llpr("Saving");
 
 		sprintf(fn, "/tmp/egon%ld", (long)getpid());
 		if (savematrix(fn, w_list->buf, NULL)) {
-			errorbox(topLevel, "Couldn't save");
+			error_box("Couldn't save");
 			return NIL;
 		}
 		sprintf(cmd, "%s %s < %s", program, param, fn);
@@ -365,6 +365,8 @@ static LISP save_external(void)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP switch_to_buffer(void)
 {
 	buffer *b;
@@ -376,12 +378,14 @@ static LISP switch_to_buffer(void)
 		b = b->next;
 		blist[nblist++] = b->name;
 	} while (b != w_list->buf);
-	if ((n = listsel(topLevel, "Change Buffer:", blist, nblist)) >= 0)
+	if ((n = select_from_list("Change Buffer:", blist, nblist)) >= 0)
 		w_list->buf = find_buffer_by_name(blist[n]);
 	activate_window(w_list);
 	return NIL;
 }
 
+/* ---
+*/
 static LISP kill_buffer(void)
 {
 	buffer *b, *next_b;
@@ -394,7 +398,7 @@ static LISP kill_buffer(void)
 		b = b->next;
 		blist[nblist++] = b->name;
 	} while (b != w_list->buf);
-	if ((n = listsel(topLevel, "Kill Buffer:", blist, nblist)) >= 0) {
+	if ((n = select_from_list("Kill Buffer:", blist, nblist)) >= 0) {
 		if ((b = find_buffer_by_name(blist[n])) != NULL) {
 			if (b != b->next) {
 				next_b = free_buffer(b);
@@ -413,12 +417,16 @@ static LISP kill_buffer(void)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP preview_block(void)
 {
 	preview();
 	return NIL;
 }
 
+/* ---
+*/
 static LISP print_block(void)
 {
 	int nr, nk;
@@ -429,6 +437,8 @@ static LISP print_block(void)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP execute_interpreter_command(LISP intpr)
 {
 	char prompt[80];
@@ -445,6 +455,8 @@ static LISP execute_interpreter_command(LISP intpr)
 
 static char *quit_buttons[] = {"Yes", "No", "Cancel"};
 
+/* ---
+*/
 static LISP lalertbox(LISP prompt, LISP buttons)
 {
 	char *btext[10], *p = get_c_string(prompt);
@@ -458,11 +470,13 @@ static LISP lalertbox(LISP prompt, LISP buttons)
 		bno++;
 	}
 	if (p == NULL) p = "prompt";
-	n = alertbox(topLevel, p, btext, bno);
+	n = alert_box(p, btext, bno);
 	while (bno) cfree(btext[--bno]);
 	return flocons(n);
 }
 
+/* ---
+*/
 static LISP quit_egon(void)
 {
 	char prompt[256], cmd[1024];
@@ -470,8 +484,7 @@ static LISP quit_egon(void)
 	do {
 		if (b->change) {
 			sprintf(prompt, "Save %s?", b->name);
-			switch (alertbox(topLevel,
-					prompt, quit_buttons, 3)) {
+			switch (alert_box(prompt, quit_buttons, 3)) {
 			case 0:
 				savematrix(b->path, b, NULL);
 				break;
@@ -493,12 +506,16 @@ static LISP quit_egon(void)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP print_version(void)
 {
-	llpr(VERSION);
+	llpr(version);
 	return NIL;
 }
 
+/* ---
+*/
 static LISP buffer_changed(LISP bname)
 {
 	buffer *buf;
@@ -510,6 +527,8 @@ static LISP buffer_changed(LISP bname)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP set_format(LISP format)
 {
 	ins_format(w_list->buf, w_list->object, w_list->script,
@@ -517,6 +536,8 @@ static LISP set_format(LISP format)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP get_format(void)
 {
 	long fmt = ret_format(w_list->buf, w_list->object, w_list->script);
@@ -526,12 +547,18 @@ static LISP get_format(void)
 /* These functions allow implementation of commands in Scheme
 	rather than in C with Scheme wrappers */
 
+/* ---
+*/
 static LISP bell(void)
 {
-	XBell(display, 100);
+#if 0
+	XBell(XtDisplay(pLevel), 100);
+#endif
 	return NIL;
 }
 
+/* ---
+*/
 static LISP lask_for_str(LISP prompt, LISP buf)
 {
 	char *p, b[256];
@@ -544,12 +571,16 @@ static LISP lask_for_str(LISP prompt, LISP buf)
 		return NIL;
 }
 
+/* ---
+*/
 static LISP lexecute(LISP cmd)
 {
 	execute(get_c_string(cmd));
 	return NIL;
 }
 
+/* ---
+*/
 static LISP linput_warp_pointer(LISP value)
 {
 	input_warp_pointer = get_c_long(value);
@@ -558,6 +589,8 @@ static LISP linput_warp_pointer(LISP value)
 
 #define MAGIC_MARKER ";;; Do not change or add anything below this line.\n"
 
+/* ---
+*/
 static LISP save_preferences(void)
 {
         char fn1[1024], fn2[1024];
@@ -587,11 +620,16 @@ static LISP save_preferences(void)
 }
 
 /* Beginning of animation functions */
+
+/* ---
+*/
 static LISP ani_c_test(void)
 {
 	return NIL;
 }
 
+/* ---
+*/
 static LISP ani_begin(void)
 {
 	buffer *b = w_list->buf;
@@ -609,78 +647,9 @@ static LISP ani_begin(void)
 	return NIL;
 }
 
-static LISP ani_property(LISP name, LISP value)
-{
-	buffer *b = w_list->buf;
-	ani_script *lasts = w_list->script;
-	ani_object *lasto = w_list->object;
-	int n = get_c_long(name);
-
-	if (!lasts) err("Last script is NULL", NIL);
-
-	if (FLONUMP(value)) {
-		long lv = get_c_long(value);
-		switch (n) {
-		case ANI_X:
-			lasts->x = lv;
-			break;
-		case ANI_Y:
-			lasts->y = lv;
-			break;
-		case ANI_WIDTH:
-			lasts->width = lv;
-			break;
-		case ANI_HEIGHT:
-			lasts->height = lv;
-			break;
-		case ANI_VISIBLE:
-			lasts->visible = lv;
-			break;
-		case ANI_COLOR:
-			lasto->color = lv;
-			break;
-		case ANI_FONT:
-			lasto->font = lv;
-			break;
-		default:
-			err("No such property", name);
-		}
-	} else {
-		char *tv = get_c_string(value);
-		switch (n) {
-		case ANI_TEXT:
-			lasto->string = cstrdup(tv);
-			break;
-		default:
-			err("No such property", name);
-		}
-	}
-	b->change = TRUE;
-	pr_scr_flag = TRUE;
-	return NIL;
-}
-
-static char *typenames[] = {
-"None", "Line", "Rectangle", "Arc", "Ellipse", "Pixmap", "String",
-"Point", "FilledRectangle", "FilledArc", "FilledEllipse", "Object"};
-
-static char *type2name(buffer *buf, int type)
-{
-	char p[256], *t;
-	ani_object *o = buf->cast;
-	int n = 0;
-
-	while (o) {
-		if (o->type == type) n++;
-		o = o->next;
-	}
-	if (type < ANI_NONE || type > ANI_FILLELLIPSE) t = "Object";
-	else t = typenames[type];
-
-	sprintf(p, "%s%d", t, n);
-	return cstrdup(p);
-}
 
+/* ---
+*/
 static LISP select_object(LISP name)
 {
 	char *string = get_c_string(name);
@@ -701,10 +670,12 @@ static LISP select_object(LISP name)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP select_tick(LISP tick)
 {
 	ani_script *s;
-	long t = get_c_long(tick);
+	int t = get_c_long(tick);
 	if (!w_list->object) {
 		llpr("No objects!");
 		return NIL;
@@ -723,6 +694,8 @@ static LISP select_tick(LISP tick)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP lani_object(LISP obj)
 {
 	buffer *b = w_list->buf;
@@ -740,7 +713,7 @@ static LISP lani_object(LISP obj)
 	}
 	w_list->object = newo;
 	newo->type = t;
-	newo->name = type2name(b, t);
+	newo->name = type2name(t);
 	lasts = newo->script = (ani_script *)cmalloc(sizeof(ani_script));
 	lasts->x = 0;
 	lasts->y = 0;
@@ -748,8 +721,7 @@ static LISP lani_object(LISP obj)
 	lasts->height = 0;
 	lasts->visible = 1;
 	lasts->time = 0;
-	newo->color = COLOR_0;
-	newo->font = HELVETICA | BOLD | SIZE_14;
+	newo->fmt = 0;
 	newo->string = NULL;
 	lasts->next = NULL;
 	w_list->script = lasts;
@@ -758,6 +730,8 @@ static LISP lani_object(LISP obj)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP set_type(LISP type)
 {
 	int t = get_c_long(type);
@@ -767,14 +741,17 @@ static LISP set_type(LISP type)
 	} else {
 		w_list->object->type = t;
 		cfree(w_list->object->name);
-		w_list->object->name = type2name(w_list->buf, t);
+		w_list->object->name = type2name(t);
 	}
 	w_list->buf->change = TRUE;
 	pr_scr_flag = TRUE;
 	return NIL;
 }
 
-/* FIXME: these two should free the memory used */
+/* ---
+FIXME: these two should free the memory used
+*/
+
 static LISP del_object(void)
 {
 	buffer *b = w_list->buf;
@@ -802,6 +779,8 @@ static LISP del_object(void)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP del_time(void)
 {
 	buffer *b = w_list->buf;
@@ -826,6 +805,8 @@ static LISP del_time(void)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP ani_background(LISP bgp)
 {
 	buffer *b = w_list->buf;
@@ -836,11 +817,13 @@ static LISP ani_background(LISP bgp)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP ani_time(LISP tim)
 {
 	ani_object *o = w_list->object;
 	ani_script *s, *news;
-	unsigned long t = get_c_long(tim);
+	unsigned int t = get_c_long(tim);
 	
 	if (!o) return NIL;
 
@@ -862,6 +845,8 @@ static LISP ani_time(LISP tim)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP lani_duration(LISP tim)
 {
 	buffer *b = w_list->buf;
@@ -870,6 +855,8 @@ static LISP lani_duration(LISP tim)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP lani_delta(LISP tim)
 {
 	buffer *b = w_list->buf;
@@ -878,6 +865,8 @@ static LISP lani_delta(LISP tim)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP lani_geometry(LISP w, LISP h)
 {
 	buffer *b = w_list->buf;
@@ -887,6 +876,8 @@ static LISP lani_geometry(LISP w, LISP h)
 	return NIL;
 }
 
+/* ---
+*/
 static LISP ani_end(void)
 {
 /*
@@ -897,22 +888,331 @@ static LISP ani_end(void)
 	return NIL;
 }
 
-static LISP lani_ctl(LISP mode, LISP now)
-{
-	int m = get_c_long(mode);
-#if 0
-	long n = get_c_long(now);
 
-	if (ani_ctl(m, n) == ANI_ERR)
-		/* perhaps it wasn't popped up? */
-		if (m == ANI_CONTINUE) return ani_end();
-#else
-	XtVaSetValues(stage, XtNanimatorMode, m, (char *)0);
-#endif
+/* ---
+*/
+static LISP lregister_x_font(LISP name, LISP x, LISP xi, LISP xb, LISP xbi)
+{
+	return flocons(register_x_font(get_c_string(name),
+					get_c_string(x),
+					get_c_string(xi),
+					get_c_string(xb),
+					get_c_string(xbi)));
+}
+
+/* ---
+*/
+static LISP lregister_ps_font(LISP name, LISP x, LISP xi, LISP xb, LISP xbi)
+{
+	return flocons(register_ps_font(get_c_string(name),
+					get_c_string(x),
+					get_c_string(xi),
+					get_c_string(xb),
+					get_c_string(xbi)));
+}
+
+/* ---
+*/
+static LISP lregister_t1_font(LISP name, LISP x, LISP xi, LISP xb, LISP xbi)
+{
+	return flocons(register_t1_font(get_c_string(name),
+					get_c_string(x),
+					get_c_string(xi),
+					get_c_string(xb),
+					get_c_string(xbi)));
+}
+
+/* ---
+*/
+static LISP lfont_alias(LISP alias, LISP name)
+{
+	return flocons(font_alias(get_c_string(alias),
+				get_c_string(name)));
+}
+
+/* ---
+*/
+static LISP lregister_color(LISP name, LISP r, LISP g, LISP b)
+{
+	return flocons(register_color(get_c_string(name),
+					get_c_long(r),
+					get_c_long(g),
+					get_c_long(b)));
+}
+
+
+/* ---
+*/
+static LISP labort(void)
+{
+	abort();
 	return NIL;
 }
 
-/* Set up the table of functions and names */
+/* ---
+Takes an associative list of the format
+((name value) (name value) ... )
+and turns it into an index into the format table.
+*/
+static LISP lencode_format(LISP f)
+{
+	sfmt fmt;
+	int result;
+
+	decode_format(0, ~0, &fmt);	/* put valid fields in fmt */
+	while (NNULLP(f)) {
+		char *name = get_c_string(CAR(CAR(f)));
+		int mask = fmt_attr2mask(name);
+		LISP value = CDR(CAR(f));
+		f = CDR(f);
+		switch (mask) {
+		case SFMT_FAMILY:
+			fmt.family = get_c_string(value);
+			break;
+		case SFMT_SIZE:
+			fmt.size = get_c_long(value);
+			break;
+		case SFMT_FG:
+			fmt.fg = get_c_string(value);
+			break;
+		case SFMT_BG:
+			fmt.bg = get_c_string(value);
+			break;
+		case SFMT_BOLD:
+			fmt.bold = get_c_long(value);
+			break;
+		case SFMT_ITALIC:
+			fmt.italic = get_c_long(value);
+			break;
+		case SFMT_ULINE:
+			fmt.uline = get_c_long(value);
+			break;
+		case SFMT_HADJ:
+			fmt.hadj = get_c_long(value);
+			break;
+		case SFMT_VADJ:
+			fmt.vadj = get_c_long(value);
+			break;
+		case SFMT_STYLE:
+			fmt.style = name2type(get_c_string(value));
+			break;
+		default:
+			break;
+		}
+		
+	}
+	result = encode_format(~0, &fmt);
+	return flocons(result);
+}
+
+static LISP SS(char *name, char *value)
+{
+	return cons(strcons(strlen(name), name),
+		    strcons(strlen(value), value));
+}
+
+static LISP SL(char *name, int value)
+{
+	return cons(strcons(strlen(name), name),
+		    flocons(value));
+}
+
+/* ---
+Takes an index into the format table and turns it into an
+associative list of the format
+((name value) (name value) ... )
+*/
+static LISP ldecode_format(LISP f)
+{
+	sfmt fmt;
+
+	decode_format(get_c_long(f), ~0, &fmt);
+	return cons(SS("family", fmt.family),
+		 cons(SL("size", fmt.size),
+		   cons(SS("fg", fmt.fg),
+		     cons(SS("bg", fmt.bg),
+		       cons(SL("bold", fmt.bold),
+			 cons(SL("italic", fmt.italic),
+			   cons(SL("uline", fmt.uline),
+			     cons(SL("hadj", fmt.hadj),
+			       cons(SL("vadj", fmt.vadj),
+				 cons(SS("style", type2name(fmt.style)),
+				      NIL))))))))));
+}
+
+/* ---
+*/
+int egon_plugin_save(int ph, char *fn)
+{
+	return plugin_save(ph, fn);
+}
+
+static LISP lplugin_register(LISP desc, LISP ext, LISP cmd)
+{
+        plugin_register(get_c_string(desc),
+                        get_c_string(ext),
+                        get_c_string(cmd));
+        return NIL;
+}
+
+static void insert_plugin(int mode, char *plugin_name)
+{
+        static char path[1024], name[1024];
+        char fn[1024];
+        char pn[1024];
+        char cmd[1024];
+        char fmt[80];
+        char *p;
+        buffer *buf = w_list->buf;
+        int n;
+        plugin_t plugin;
+
+        /* ask for file name */
+        if (path[0] == '\0') getcwd(path, 1024);
+        name[0] = fn[0] = '\0';
+
+        if (plugin_name) {
+                char *q = strrchr(plugin_name, '/');
+                if (q) strcpy(name, q+1);
+                else strcpy(name, plugin_name);
+                strcpy(fn, plugin_name);
+        } else {
+	        if (!select_file(path, name, plugin_patterns, fmt)) return;
+        	sprintf(fn, "%s/%s", path, name);
+	}
+        plugin_unique_name(name, pn);
+
+        plugin.row = 1;
+        plugin.col = 1;
+        plugin.displayed = 0;
+
+        if (mode == PLUGIN_COPY) {
+                plugin.name = cstrdup(pn);
+                /* copy the file */
+                p = plugin_basedir(buf, NULL);
+                sprintf(pn, "%s/%s", p, plugin.name);
+                sprintf(cmd, "(mkdir %s;cp %s %s)2>/dev/null", p, fn, pn);
+                system(cmd);
+        } else {
+                strcpy(pn, fn);
+                plugin.name = cstrdup(pn);
+        }
+
+        /* start the plugin */
+        plugin.ph = plugin_start(pn);
+        if (plugin.ph == -1) return;
+
+        /* prepare the buffer */
+        n = buf->nplugin++;
+        buf->plugin = (plugin_t *)crealloc(buf->plugin,
+                buf->nplugin*sizeof(plugin_t));
+        buf->plugin[n] = plugin;
+        buf->change = TRUE;
+        pr_scr_flag = TRUE;
+}
+
+static LISP lplugin_import(LISP pn)
+{
+        if (NULLP(pn)) insert_plugin(PLUGIN_COPY, NULL);
+        else insert_plugin(PLUGIN_COPY, get_c_string(pn));
+        return NIL;
+}
+
+/* ---
+broken because it still takes the name relative to basedir
+*/
+
+static LISP lplugin_link(LISP pn)
+{
+        if (NULLP(pn)) insert_plugin(PLUGIN_LINK, NULL);
+        else insert_plugin(PLUGIN_LINK, get_c_string(pn));
+        return NIL;
+}
+
+/* ---
+return buffer's plug handler index or -1 for error
+*/
+
+static int select_plugin(buffer *b)
+{
+        char *plugin[20];
+        int i;
+
+        if (b->nplugin == 0) return -1;
+        if (b->nplugin == 1) return 0;
+        for (i = 0; i < 20 && i < b->nplugin; i++)
+                plugin[i] = b->plugin[i].name;
+        i = select_from_list("Select Plugin:", plugin, i);
+        return i;
+}
+
+static LISP lplugin_select(void)
+{
+	return flocons(select_plugin(w_list->buf));
+}
+
+static LISP lplugin_export(void)
+{
+        int n;
+        char fn[1024], pn[1024], cmd[1024], path[1024], fmt[80];
+        buffer *buf = w_list->buf;
+
+        /* pick plugin from list */
+        n = select_plugin(buf);
+        if (n == -1) return NIL;
+
+        /* get description of the handler */
+        /* ask for file name */
+        getcwd(path, 1024);
+        strcpy(fn, buf->plugin[n].name);
+        if (!select_file(path, fn, NULL, fmt)) return NIL;
+
+        /* save the file */
+        sprintf(pn, "%s/%s", plugin_basedir(buf, NULL), buf->plugin[n].name);
+        sprintf(cmd, "cp %s %s/%s", pn, path, fn);
+        system(cmd);
+        return NIL;
+}
+
+static LISP lplugin_delete(void)
+{
+        int n;
+        buffer *buf = w_list->buf;
+
+        /* pick plugin from list */
+        n = select_plugin(buf);
+        if (n == -1) return NIL;
+
+        /* remove plugin */
+        plugin_stop(buf->plugin[n].ph);
+        cfree(buf->plugin[n].name);
+        buf->nplugin--;
+        for (; n < buf->nplugin; n++)
+                buf->plugin[n] = buf->plugin[n+1];
+        buf->change = pr_scr_flag = TRUE;
+        return NIL;
+}
+
+static LISP lplugin_move(void)
+{
+        int n;
+        buffer *buf = w_list->buf;
+
+        /* pick plugin from list */
+        n = select_plugin(buf);
+        if (n == -1) return NIL;
+
+        buf->plugin[n].row = 1;
+        buf->plugin[n].col = 1;
+        buf->change = pr_scr_flag = TRUE;
+        return NIL;
+}
+
+static LISP lmake_backups(LISP p)
+{
+	make_backups = get_c_long(p);
+	return NIL;
+}
 
 /* Commands that take no arguments */
 s_fn_table fn_table[] =
@@ -925,7 +1225,7 @@ s_fn_table fn_table[] =
 	/* editing */
 
 	/* block commands */
-	{"preview-block", preview_block},
+	{"preview", preview_block},
 	{"print-block", print_block},
 
 	/* new window */
@@ -947,7 +1247,7 @@ s_fn_table fn_table[] =
 	/* help commands */
 	{"print-version", print_version},
 
-	{"quit-egon", quit_egon},
+	{"quit-program", quit_egon},
 	/* low level functions */
 	{"bell", bell},
 
@@ -958,6 +1258,7 @@ s_fn_table fn_table[] =
 	{"ani-c-test", ani_c_test},
 	{"get-format", get_format},
 	{"save-preferences", save_preferences},
+	{"abort", labort},
 	{(char *) 0, (LISP(*)())0}
 };
 
@@ -980,6 +1281,9 @@ s_fn_table fn_table1[] = {
 	{"select-tick", select_tick},
 	{"set-type", set_type},
 	{"set-format", set_format},
+	{"decode-format", ldecode_format},
+	{"encode-format", lencode_format},
+	{"make-backups", lmake_backups},
 	{NULL, NULL}
 };
 
@@ -988,12 +1292,15 @@ s_fn_table fn_table2[] = {
 	{"ask-for-str", lask_for_str},
 	{"alertbox", lalertbox},
 
-	{"ani-property", ani_property},
 	{"ani-geometry", lani_geometry},
-	{"ani-ctl", lani_ctl},
+	{"font-alias", lfont_alias},
 	{NULL, NULL}
 };
 
+/* ---
+Set up the table of functions and names
+*/
+
 void init_cmds(void)
 {
 	int i;
@@ -1004,5 +1311,16 @@ void init_cmds(void)
 		init_subr_1(fn_table1[i].name, fn_table1[i].function);
 	for (i = 0; fn_table2[i].name; i++)
 		init_subr_2(fn_table2[i].name, fn_table2[i].function);
+	init_subr_5("register-x-font", lregister_x_font);
+	init_subr_5("register-ps-font", lregister_ps_font);
+	init_subr_5("register-t1-font", lregister_t1_font);
+	init_subr_4("register-color", lregister_color);
+	init_subr_3("plugin-register", lplugin_register);
+	init_subr_0("plugin-select", lplugin_select);
+        init_subr_1("plugin-import", lplugin_import);
+        init_subr_0("plugin-export", lplugin_export);
+        init_subr_1("plugin-link", lplugin_link);
+        init_subr_0("plugin-delete", lplugin_delete);
+        init_subr_0("plugin-move", lplugin_move);
 }
 
