@@ -14,7 +14,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+   MA 02111-1307, USA.
  */
 
 /*
@@ -26,22 +27,19 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <X11/Xlib.h>
-#include <X11/Intrinsic.h>
-#include <X11/xpm.h>
 
-#include "../xcommon/Animator.h"
 #include "egon.h"
 #include "types.h"
 #include "../common/common.h"
-#include "../common/fonts.h"
-#include "../xcommon/embed.h"
 
-/* format guessing:
+#define PS_MAGIC "%!PS"
+
+/* ---
+format guessing:
    1. extension .ps
    2. Starts with "%!PS"
 */
-#define PS_MAGIC "%!PS"
+
 static int myformat(char *fn)
 {
 	char *ext;
@@ -58,6 +56,8 @@ static int myformat(char *fn)
 	return result;
 }
 
+/* ---
+*/
 void fileio_ps_init()
 {
 	register_format(NULL, NULL, myformat, "Postscript (*.ps)");
