@@ -1,3 +1,23 @@
+/*
+   Pathetic Writer
+   Copyright (C) 1997, 1998  Ulric Eriksson <ulric@edu.stockholm.se>
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+   MA 02111-1307, USA.
+ */
+
 /* user_interface.h
 
    These are all the functions that the user interface must implement
@@ -7,9 +27,6 @@
    This file is included from calc.h and from [xt]siag.h.
 */
 
-/*extern Window xwindow_of_window(window *);*/
-
-extern void interp_startup(void);
 extern int select_file(char *, char *, char **, char *);
 extern int alert_box(char *, char **, int);
 extern int select_from_list(char *, char **, int);
@@ -31,10 +48,3 @@ extern textbuf kbd_macro;
 
 extern spell_state spell_select(char *, char *);
 
-/* plugin functions */
-extern int pw_plugin_start(char *);
-extern int pw_plugin_stop(int);
-extern int pw_plugin_save(int, char *);
-extern int pw_plugin_print(int, FILE *);
-extern int pw_plugin_size_get(int, unsigned long *, unsigned long *);
-
