@@ -28,7 +28,6 @@
 #include <ctype.h>
 
 #include "calc.h"
-#include "types.h"
 
 static int load_scm(char *fn, buffer *buf)
 {
@@ -40,14 +39,18 @@ static int load_scm(char *fn, buffer *buf)
 	return 0;
 }
 
-/* conservative file format guessing:
+/* ---
+conservative file format guessing:
    Never match, it's too dangerous
 */
+
 static int myformat(char *fn)
 {
 	return 0;
 }
 
+/* ---
+*/
 void fileio_scm_init(void)
 {
 	register_format(load_scm, NULL, myformat,
