@@ -1,6 +1,6 @@
 /*
    Siag, Scheme In A Grid
-   Copyright (C) 1996-1998  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1996-1999  Ulric Eriksson <ulric@edu.stockholm.se>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,10 @@
    MA 02111-1307, USA.
  */
 
-/* 980727: WK1 format support by Jon K. Hellan" <Jon.K.Hellan@item.ntnu.no> */
+/* ---
+   980727: WK1 format support by Jon K. Hellan" <Jon.K.Hellan@item.ntnu.no>
+   981030: Dirty hack to make this work with the new format code.
+--- */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,10 +31,10 @@
 #include "../common/common.h"
 #include "../common/cmalloc.h"
 #include "../common/fonts.h"
-#include "types.h"
 #include "calc.h"
 
-/* Saving strategy:
+/* ---
+Saving strategy:
    We always have the fallback option to save as a floating-point
    or string constant. But it is desirable to save formulas as such
    to not lose information. In particular, a wk1 file that can be
@@ -40,7 +43,7 @@
    read-from-string function. The resulting list is then passed
    into a C function, which traverses it and generates RPN code
    to be stored in the worksheet.
-*/
+--- */
 
 static LISP pre2post(LISP l)
 {
@@ -68,13 +71,13 @@ static LISP pre2post(LISP l)
 	return NIL;
 }
 
-static int col_last_changed(buffer *b, int row)
+static int col_last_changed(buffer *b, int s, int row)
 {
         int i;
 
-        if (row > b->alloc_lines) return 0;
-        for (i = b->alloc_cols[row]; i > 1; i--)
-                if (ret_type(b, row, i) != EMPTY)
+        if (row > b->sht[s].alloc_lines) return 0;
+        for (i = b->sht[s].alloc_cols[row]; i > 1; i--)
+                if (ret_type(b, s, row, i) != EMPTY)
                         break;
         return i;
 }
@@ -126,16 +129,16 @@ static void writefloat(unsigned char *p, double x)
         }
 }
 
-static int save_cell(FILE *fp, buffer *buf, long r, long c)
+static int save_cell(FILE *fp, buffer *buf, int sh, int r, int c)
 {
 	unsigned char b[2000], s[2000], *p;
 	double num;
 	int len;
 
-	switch (ret_type(buf, r, c)) {
+	switch (ret_type(buf, sh, r, c)) {
 	case LABEL:
 		writeint(b, 0x0F);		/* type LABEL */
-		p = (unsigned char *)ret_text(buf, r, c);
+		p = (unsigned char *)ret_text(buf, sh, r, c);
 		len = 1+2+2+1+strlen((char *)p)+1;
 		writeint(b+2, len);
 		b[4] = 0xFF;			/* format */
@@ -147,7 +150,7 @@ static int save_cell(FILE *fp, buffer *buf, long r, long c)
 		break;
 	case STRING:	/* treat as label */
 		writeint(b, 0x0F);		/* type LABEL */
-		p = (unsigned char *)ret_pvalue(NULL, buf, r, c, -1);
+		p = (unsigned char *)ret_pvalue(NULL, buf, sh, r, c, -1);
 		len = 1+2+2+1+strlen((char *)p)+1;
 		writeint(b+2, len);
 		b[4] = 0xFF;			/* format */
@@ -164,9 +167,9 @@ static int save_cell(FILE *fp, buffer *buf, long r, long c)
 		b[4] = 0xFF;			/* format */
 		writeint(b+5, c-1);		/* column */
 		writeint(b+7, r-1);		/* row */
-		num = ret_val(buf, r, c).number;
+		num = ret_val(buf, sh, r, c).number;
 		writefloat(b+9, num);
-		sprintf((char *)s, "(scm->list %ld %ld)", r, c);
+		sprintf((char *)s, "(scm->list %d %d)", r, c);
 		execute((char *)s);
 		fwrite(b, 1, len+4, fp);	/* 4 is for type and length */
 		break;
@@ -178,12 +181,13 @@ static int save_cell(FILE *fp, buffer *buf, long r, long c)
 
 static int save_wk1(char *fn, buffer *buf)
 {
-	long r, c;
+	int r, c;
 	FILE *fp;
 	static unsigned char preblurb[] =
 		{0x00, 0x00, 0x02, 0x00, 0x06, 0x04};
 	static unsigned char postblurb[] =
 		{0x01, 0x00, 0x00, 0x00};
+	int s = 0;	/* only first sheet */
 
 	if ((fp = fopen(fn, "wb")) == NULL)
 		return 1;
@@ -192,9 +196,9 @@ static int save_wk1(char *fn, buffer *buf)
 		fclose(fp);
 		return 1;
 	}
-	for (r = 1; r <= line_last_used(buf); r++) {
-		for (c = 1; c <= col_last_changed(buf, r); c++) {
-			if (save_cell(fp, buf, r, c)) {
+	for (r = 1; r <= line_last_used(buf, s); r++) {
+		for (c = 1; c <= col_last_changed(buf, s, r); c++) {
+			if (save_cell(fp, buf, s, r, c)) {
 				fclose(fp);
 				return 1;
 			}
@@ -208,14 +212,15 @@ static int save_wk1(char *fn, buffer *buf)
 	return 0;
 }
 
-/* Loading strategy:
+/* ---
+Loading strategy:
    The Lotus format stores formulas in postfix notation. It is most
    straightforward to translate this to prefix, i.e. Scheme.
    Every token is stored on a stack, in string form. Example:
    1 2 + => "1", "1" "2", "(+ 1 2)"
    So an operator is done by creating a new string from several on
    the stack.
-*/
+--- */
 
 static char *stack[10];
 static int sp;
@@ -233,22 +238,28 @@ static char *pop(void)
 	return stack[sp];
 }
 
-/* Dates in wk1 format are based on day 0 = Jan 1st 1900, However, 
+/* ---
+Dates in wk1 format are based on day 0 = Jan 1st 1900, However, 
    files produced by Excel are 1 day off, because Excel believes that 1900
    was a leap year. I don't know if Lotus has the same bug, and choose to
    treat Excel as the norm. 
    BTW, this code breaks in 2034, if we are still using 32 bit int then.
-   Jon Kre Hellan July 23rd 1998 */
+   Jon Kre Hellan July 23rd 1998
+*/
+
 static double from_wk1date(const double wk1date, int justtime)
 {
 	double rdate = wk1date;
 	if (!justtime) {
 		rdate -= 25569;
 	}
-	return (double)(rdate * 84600);
+	return (double)(rdate * 86400);
 }
 
-/* borrowed from teapot */
+/* ---
+borrowed from teapot
+*/
+
 static double readfloat(const unsigned char *s)
 {
   double x;
@@ -377,7 +388,6 @@ static int readfmt(unsigned char *p)
 	return format;
 }
 
-
 static int makecoord(int orig, int coord)
 {
         if (coord & 0x8000)
@@ -453,18 +463,33 @@ static void formula(int len, unsigned char *p, int r, int c)
 			i += 9;
 			break;
 		case 0x01:
-			sprintf(b, "r%dc%d",
-                                1+makecoord(r, readint(p+i+3)),
-                                1+makecoord(c, readint(p+i+1)));
+			if (a1_refs == 0) {
+				sprintf(b, "r%dc%d",
+                        	        1+makecoord(r, readint(p+i+3)),
+                        	        1+makecoord(c, readint(p+i+1)));
+			} else {
+				sprintf(b, "%s%d",
+       	                         	a1coord(1+makecoord(c, readint(p+i+1))),
+       	                         	1+makecoord(r, readint(p+i+3)));
+			}
 			push(cstrdup(b));
 			i += 5;
 			break;
 		case 0x02:
-			sprintf(b, "r%dc%d..r%dc%d",
-                                1+makecoord(r, readint(p+i+3)),
-                                1+makecoord(c, readint(p+i+1)),
-                                1+makecoord(r, readint(p+i+7)),
-                                1+makecoord(c, readint(p+i+5)));
+			if (a1_refs == 0) {
+				sprintf(b, "r%dc%d..r%dc%d",
+	                                1+makecoord(r, readint(p+i+3)),
+	                                1+makecoord(c, readint(p+i+1)),
+	                                1+makecoord(r, readint(p+i+7)),
+	                                1+makecoord(c, readint(p+i+5)));
+			} else {
+				sprintf(b, "%s%d",
+       	                         	a1coord(1+makecoord(c, readint(p+i+1))),
+                                	1+makecoord(r, readint(p+i+3)));
+				sprintf(b+strlen(b), "..%s%d",
+                                	a1coord(1+makecoord(c, readint(p+i+5))),
+                                	1+makecoord(r, readint(p+i+7)));
+			}
 			push(cstrdup(b));
 			i += 9;
 			break;
@@ -512,7 +537,7 @@ static void formula(int len, unsigned char *p, int r, int c)
 		CASE1(0x21, "abs", 1);
 		CASE1(0x22, "floor", 1);
 		CASE1(0x23, "sqrt", 1);
-		CASE1(0x24, "log10", 1);
+		CASE1(0x24, "@LOG10", 1);
 		CASE1(0x25, "log", 1);
 		CASE0(0x26, "@PI", 1);
 		CASE1(0x27, "sin", 1);
@@ -607,10 +632,12 @@ static void formula(int len, unsigned char *p, int r, int c)
 
 static void wk1(FILE *fp, buffer *buf)
 {
-        int i, j, sf = SIZE_10 | HELVETICA, f, rf;
+        int i, j, sf = fmt_new2old(0), f, rf;
 	char b[2000];
 	char *p1;
 	cval value;
+	int s = 0;	/* first sheet */
+	buf->a1_refs = a1_refs;	/* imported doc will use A1 references */
 
 	for (;;) {
 		unsigned char head[4];
@@ -653,8 +680,8 @@ static void wk1(FILE *fp, buffer *buf)
 				sprintf(b, "%d", (int)value.number);
 			}
 			ins_data(buf, siod_interpreter, b,
-				value, EXPRESSION, i, j);
-			ins_format(buf,	i, j, f);
+				value, EXPRESSION, s, i, j);
+			ins_format(buf,	s, i, j, fmt_old2new(f));
 			break;
 		case 0x0E:
 			value.number = readfloat(p+5);
@@ -669,8 +696,8 @@ static void wk1(FILE *fp, buffer *buf)
 				sprintf(b, "%d", (int) value.number);
 			}
 			ins_data(buf, siod_interpreter, b,
-				value, EXPRESSION, i, j);
-			ins_format(buf,	i, j, f);
+				value, EXPRESSION, s, i, j);
+			ins_format(buf,	s, i, j, fmt_old2new(f));
 			break;
 		case 0x0F:
 			switch (p[5]) {
@@ -688,8 +715,8 @@ static void wk1(FILE *fp, buffer *buf)
 			j = 1+readint(p+1);
 			f = sf | readfmt(p); 
 			ins_data(buf, siod_interpreter, (char *)p+6,
-				value, LABEL, i, j);
-			ins_format(buf,	i, j, f);
+				value, LABEL, s, i, j);
+			ins_format(buf,	s, i, j, fmt_old2new(f));
 			break;
 		case 0x10:
 		/* 5-12 = value */
@@ -701,9 +728,9 @@ static void wk1(FILE *fp, buffer *buf)
 			p1 = pop();
 			value.number = readfloat(p+5);
 			ins_data(buf, siod_interpreter, p1,
-				value, EXPRESSION, i+1, j+1);
+				value, EXPRESSION, s, i+1, j+1);
 			cfree(p1);
-			ins_format(buf,	i+1, j+1, f);
+			ins_format(buf,	s, i+1, j+1, fmt_old2new(f));
 			break;
 		default:
 			break;
@@ -723,10 +750,12 @@ static int load_wk1(char *fn, buffer *buf)
 	return 0;
 }
 
-/* File format guessing:
+/* ---
+File format guessing:
    1. extension .wk1 or .wks
    2. starts with the sequence 0x00 0x00 0x02 0x00
 */
+
 static int myformat(char *fn)
 {
         char *ext;
@@ -750,6 +779,8 @@ static int myformat(char *fn)
         return 0;
 }
 
+/* ---
+*/
 void fileio_wk1_init(void)
 {
 	init_subr_1("pre->post", pre2post);
