@@ -1,43 +1,33 @@
 ;;
 ;; Runtime library for Siag
-;; Copyright 1996-1998 Ulric Eriksson
+;; Copyright 1996-1999 Ulric Eriksson
 ;;
 
 ; Load the runtime library from Siod
-(require (string-append SIAGHOME "/siod/siod.scm"))
+(require (string-append datadir "/siod/siod.scm"))
 
-; These are missing
-(define (caaaar l) (caar (caar l)))
-(define (cdaaar l) (cdar (caar l)))
-(define (cadaar l) (cadr (caar l)))
-(define (cddaar l) (cddr (caar l)))
-(define (caadar l) (caar (cdar l)))
-(define (cdadar l) (cdar (cdar l)))
-(define (caddar l) (cadr (cdar l)))
-(define (cdddar l) (cddr (cdar l)))
-(define (caaadr l) (caar (cadr l)))
-(define (cdaadr l) (cdar (cadr l)))
-(define (cadadr l) (cadr (cadr l)))
-(define (cddadr l) (cddr (cadr l)))
-(define (caaddr l) (caar (cddr l)))
-(define (cdaddr l) (cdar (cddr l)))
-(define (cadddr l) (cadr (cddr l)))
-(define (cddddr l) (cddr (cddr l)))
+; Load Common
+(require (string-append datadir "/common/common.scm"))
+(require (string-append datadir "/common/position.scm"))
 
 ; Set up the menu
-(require (string-append SIAGHOME "/siag/menu.scm"))
+(define (create-menus)
+  (load (string-append datadir "/siag/menu.scm"))
+  (load (string-append datadir "/siag/toolbar.scm")))
 
 ; Set up the key bindings
-(require (string-append SIAGHOME "/siag/keytable.scm"))
+(define (init-windows)
+  (require (string-append datadir "/siag/keytable.scm"))
+  (require (string-append datadir "/plugins/plugin.scm")))
 
 ; X string definitions
-(require (string-append SIAGHOME "/xcommon/StringDefs.scm"))
+(require (string-append datadir "/xcommon/StringDefs.scm"))
 
 ; Form interface
-(require (string-append SIAGHOME "/xcommon/form.scm"))
+(require (string-append datadir "/xcommon/form.scm"))
 
 ; Data interface
-(require (string-append SIAGHOME "/siag/data.scm"))
+(require (string-append datadir "/siag/data.scm"))
 
 (define EMPTY 0)
 (define LABEL 1)
@@ -46,18 +36,8 @@
 (define STRING 5)
 (define EMBED 6)
 
-(define (llpr x)
-	(puts x))
-
-;; Handle the "position"
-(define (make-position row col)
-	(list row col))
-
-(define (position-row p)
-	(car p))
-
-(define (position-col p)
-	(cadr p))
+(setq P-MIN (make-position 1 1))
+(setq P-MAX (make-position 100000 100000))
 
 ;; Functions that do approximately the same as position.c
 ;;
@@ -66,99 +46,57 @@
 ;; get-blku, set-blku, get-prot, set-prot
 ;; line-last-used, col-last-used, max-lines, max-columns
 
-(define (get-point-row)
-  (position-row (get-point)))
-
-(define (get-point-col)
-  (position-col (get-point)))
-
-(define (set-point-row row)
-  (set-point (make-position row (get-point-col))))
-
-(define (set-point-col col)
-  (set-point (make-position (get-point-row) col)))
-
 (define (find-beginning-of-buffer)
   (get-prot))
 
 (define (find-end-of-buffer)
-	(let ((row (line-last-used)))
-		(make-position row (col-last-used row))))
+  (let ((row (line-last-used)))
+    (make-position row (col-last-used row))))
 
-(define (line-forward p)
-	(if (< (position-row p) (max-lines))
-		(make-position (+ (position-row p) 1) (position-col p))
-		p))
-
-(define (line-backward p)
-	(if (> (position-row p) (position-row (get-prot)))
-		(make-position (- (position-row p) 1) (position-col p))
-		p))
-
-(define (cell-forward p)
-	(if (< (position-col p) (max-columns))
-		(make-position (position-row p) (+ (position-col p) 1))
-		p))
-
-(define (cell-backward p)
-	(if (> (position-col p) 1)
-		(make-position (position-row p) (- (position-col p) 1))
-		p))
+; Find a position not out of bounds
+(define (safe-position row col)
+  (let* ((top (find-beginning-of-buffer))
+         (top-r (position-row top))
+         (top-c (position-col top)))
+    (if (< row top-r) (set! row top-r))
+    (if (> row (position-row P-MAX)) (set! row (position-row P-MAX)))
+    (if (< col top-c) (set! col top-c))
+    (if (> col (position-col P-MAX)) (set! col (position-col P-MAX))))
+  (make-position row col))
 
 ;; And now: commands written in Scheme
-(define (backward-cell)
-	(set-point (cell-backward (get-point))))
-
-(define (forward-cell)
-	(set-point (cell-forward (get-point))))
-
-
-(define (set-mark-command)
-	(let ()
-		(set-mark (get-point))
-		(llpr "Mark set")))
-
-(define (exchange-point-and-mark)
-	(let ((p (get-point)))
-		(set-point (get-mark))
-		(set-mark p)))
-
-(define (beginning-of-buffer)
-	(set-mark (get-point))
-	(set-point (find-beginning-of-buffer)))
-
-(define (end-of-buffer)
-	(set-mark (get-point))
-	(set-point (find-end-of-buffer)))
-
-(define (top-of-buffer)
-	(set-point-row 1))
-
-(define (bottom-of-buffer)
-	(set-point-row (line-last-used)))
 
 (define (beginning-of-line)
-	(set-point-col 1))
+	(set-point-col (position-col (find-beginning-of-buffer))))
 
 (define (end-of-line)
 	(set-point-col (col-last-used (get-point-row))))
 
-(define (next-line)
-	(set-point (line-forward (get-point))))
-
-(define (previous-line)
-	(set-point (line-backward (get-point))))
-
-(setq BUFFER-ROWS 100000)
-(setq BUFFER-COLS 100000)
-
 (define (select-all)
 	(let ((p (get-point)))
-		(set-mark (make-position 1 1))
-		(set-point (make-position BUFFER-ROWS BUFFER-COLS))
+		(set-mark P-MIN)
+		(set-point P-MAX)
 		(set-block)
 		(set-point p)))
 
+(define (top-of-buffer)
+  (set-point-row (position-row (find-beginning-of-buffer))))
+
+(define (bottom-of-buffer)
+  (set-point-row (line-last-used)))
+
+(define (forward-cell)
+  (set-point (next-col (get-point))))
+
+(define (backward-cell)
+  (set-point (previous-col (get-point))))
+
+(define (next-line)
+  (set-point (next-row (get-point))))
+
+(define (previous-line)
+  (set-point (previous-row (get-point))))
+
 ;;
 ;; A few shorthand functions for my convenience
 ;;
@@ -181,33 +119,14 @@
 ;; Some more commands
 ;;
 
-(define (execute-extended-command)
-  (execute-interpreter-command 'SIOD))
+(define (quit-siag)
+  (quit-program))
 
-(define (new-siag)
+(define (new-buffer)
   (spawn "siag"))
 
-(define (do-help helpfile)
-  (spawn (string-append SIAGHELP " file:"
-	 SIAGDOCS "/" helpfile)))
-
 (define (help-contents)
-  (do-help "siag/docs/siag.html"))
-
-(define (help-copyright)
-  (do-help "siag/docs/COPYING"))
-
-(define (help-for-help)
-  (do-help "common/docs/siaghelp.html"))
-
-(define (help-search)
-  (do-help "common/docs/search.html"))
-
-(define (keyboard-quit)
-  (llpr "Quit"))
-
-(define (no-op)
-  (llpr "This command does nothing"))
+  (do-help "siag/siag.html"))
 
 (define (get-cell-coords top point)
   (let ((top-row (position-row top))
@@ -261,9 +180,7 @@
     (while (< new-y 20)
       (set! new-y (+ new-y 20))
       (set! new-row (- new-row 1)))
-    (if (< new-row 1)
-      (set! new-row 1))
-    (set-top (make-position new-row (position-col top)))
+    (set-top (safe-position new-row (position-col top)))
     (fix-point width height)
     (set-pr-scr)))
 
@@ -278,9 +195,7 @@
     (while (< new-y 0)
       (set! new-y (+ new-y 20))
       (set! new-row (- new-row 1)))
-    (if (< new-row 1)
-      (set! new-row 1))
-    (set-top (make-position new-row (position-col top)))
+    (set-top (safe-position new-row (position-col top)))
     (fix-point width height)
     (set-pr-scr)))
 
@@ -295,9 +210,7 @@
     (while (> new-y 0)
       (set! new-y (- new-y 20))
       (set! new-row (+ new-row 1)))
-    (if (> new-row BUFFER-ROWS)
-      (set! new-row BUFFER-ROWS))
-    (set-top (make-position new-row (position-col top)))
+    (set-top (safe-position new-row (position-col top)))
     (fix-point width height)
     (set-pr-scr)))
 
@@ -312,9 +225,7 @@
     (while (> new-y 0)
       (set! new-y (- new-y 20))
       (set! new-row (+ new-row 1)))
-    (if (> new-row BUFFER-ROWS)
-      (set! new-row BUFFER-ROWS))
-    (set-top (make-position new-row (position-col top)))
+    (set-top (safe-position new-row (position-col top)))
     (fix-point width height)
     (set-pr-scr)))
 
@@ -329,9 +240,7 @@
     (while (< new-x 0)
       (set! new-x (+ new-x 80))
       (set! new-col (- new-col 1)))
-    (if (< new-col 1)
-      (set! new-col 1))
-    (set-top (make-position (position-row top) new-col))
+    (set-top (safe-position (position-row top) new-col))
     (fix-point width height)
     (set-pr-scr)))
 
@@ -346,9 +255,7 @@
     (while (< new-x 0)
       (set! new-x (+ new-x 80))
       (set! new-col (- new-col 1)))
-    (if (< new-col 1)
-      (set! new-col 1))
-    (set-top (make-position (position-row top) new-col))
+    (set-top (safe-position (position-row top) new-col))
     (fix-point width height)
     (set-pr-scr)))
 
@@ -363,9 +270,7 @@
     (while (> new-x 0)
       (set! new-x (- new-x 80))
       (set! new-col (+ new-col 1)))
-    (if (> new-col BUFFER-COLS)
-      (set! new-col BUFFER-COLS))
-    (set-top (make-position (position-row top) new-col))
+    (set-top (safe-position (position-row top) new-col))
     (fix-point width height)
     (set-pr-scr)))
 
@@ -380,9 +285,7 @@
     (while (> new-x 0)
       (set! new-x (- new-x 80))
       (set! new-col (+ new-col 1)))
-    (if (> new-col BUFFER-COLS)
-      (set! new-col BUFFER-COLS))
-    (set-top (make-position (position-row top) new-col))
+    (set-top (safe-position (position-row top) new-col))
     (fix-point width height)
     (set-pr-scr)))
 
@@ -449,7 +352,7 @@
 
 (define *cut-buffer* nil)
 
-(define (copy-block)
+(define (copy-block-to-string)
   (set! *cut-buffer* (pack-area (position-row (get-blku))
 				(position-col (get-blku))
 				(position-row (get-blkl))
@@ -458,11 +361,11 @@
   (calc-matrix nil)
   (set-pr-scr))
 
-(define (cut-block)
-  (copy-block)
+(define (cut-block-to-string)
+  (copy-block-to-string)
   (delete-block))
 
-(define (paste-block)
+(define (paste-block-from-string)
   (unpack-area *cut-buffer* (get-point-row) (get-point-col))
   (buffer-changed nil)
   (calc-matrix nil)
@@ -691,15 +594,20 @@
 ; Load the plotting functions
 ; Don't load until actually needed
 (define (plot . args)
-  (require (string-append SIAGHOME "/siag/plot.scm"))
+  (require (string-append datadir "/siag/plot.scm"))
   (apply plot args))
 
+(define (plot-plugin . args)
+  (require (string-append datadir "/siag/plot.scm"))
+  (apply plot-plugin args))
+
 (define (splot . args)
-  (require (string-append SIAGHOME "/siag/splot.scm"))
+  (require (string-append datadir "/siag/splot.scm"))
   (apply splot args))
 
-(define viewer-command "gv")
-(define lpr-command "lpr")
+(define (plot-wizard)
+  (require (string-append datadir "/siag/plot.scm"))
+  (plot-wizard2))
 
 (define (preview)
   (let ((fn (string-append "/tmp/siagprint" (number->string (getpid)) ".ps")))
@@ -729,31 +637,9 @@
   (set-pr-scr))
 
 (define (remove-protection)
-  (set-prot (make-position 1 1))
+  (set-prot P-MIN)
   (set-pr-scr))
 
-(define BLACK 0)
-(define RED 1)
-(define GREEN 2)
-(define BLUE 3)
-(define YELLOW 4)
-(define MAGENTA 5)
-(define CYAN 6)
-(define WHITE 7)
-
-(define *colors*
-  (list (cons "black" BLACK)
-	(cons "red" RED)
-	(cons "green" GREEN)
-	(cons "blue" BLUE)
-	(cons "yellow" YELLOW)
-	(cons "magenta" MAGENTA)
-	(cons "cyan" CYAN)
-	(cons "white" WHITE)))
-
-(define (string->color color)
-  (or (cdr (assoc color *colors*)) BLACK))
-
 (define (ask-for-color)
   (form-begin)
   (form-label "Color:")
@@ -770,24 +656,6 @@
   (form-property XtNwidth 80)
   (cdr (assoc "color" (form-end))))
 
-(define (cell-color)
-  (set-color nil
-	     (get-point)
-	     (bit-and 7 (string->color (ask-for-color)))))
-
-(define (block-color)
-  (let ((blku (get-blku nil))
-	(blkl (get-blkl nil))
-	(color (bit-and 7 (string->color (ask-for-color))))
-	(row 0)
-	(col 0))
-    (set! row (position-row blku))
-    (while (<= row (position-row blkl))
-      (set! col (position-col blku))
-      (while (<= col (position-col blkl))
-	(set-color nil (make-position row col) color)
-	(set! col (+ col 1)))
-      (set! row (+ row 1)))))
 
 ; A hack to allow simple hyperlinking.
 ; Calls execute on the text in the next cell.
@@ -795,52 +663,13 @@
   (execute (get-text (row) (+ (col) 1))))
 
 (define (search-forward)
-  (require (string-append SIAGHOME "/siag/find.scm"))
+  (require (string-append datadir "/siag/find.scm"))
   (search-forward))
 
 (define (search-backward)
-  (require (string-append SIAGHOME "/siag/find.scm"))
+  (require (string-append datadir "/siag/find.scm"))
   (search-backward))
 
-; (change-format fromr tor fromc toc format mask)
-; format = new format 
-; mask = which bits to change
-(define (change-format fromr tor fromc toc format mask)
-  (set! tor (min tor (line-last-used)))
-  (while (<= fromr tor)
-    (let ((c fromc)
-	  (c2 (min toc (col-last-used fromr))))
-      (while (<= c c2)
-	(let ((oldfmt (get-format nil (make-position fromr c))))
-	  (set-format nil (make-position fromr c)
-			  (bit-or (bit-and format mask)
-				  (bit-and oldfmt (bit-not mask)))))
-	(set! c (+ c 1))))
-    (set! fromr (+ fromr 1)))
-  (set-pr-scr))
-
-(define (in-block point blku blkl)
-  (and (>= (position-row point) (position-row blku))
-       (<= (position-row point) (position-row blkl))
-       (>= (position-col point) (position-col blku))
-       (<= (position-col point) (position-col blkl))))
-
-(define FONT_MASK 96)
-(define COURIER 0)
-(define HELVETICA 32)
-(define NEW_CENTURY 64)
-(define TIMES 96)
-
-(define SIZE_MASK 7)
-(define SIZE_8 0)
-(define SIZE_10 1)
-(define SIZE_12 2)
-(define SIZE_14 3)
-(define SIZE_18 4)
-(define SIZE_24 5)
-(define SIZE_20 6)
-(define SIZE_30 7)
-
 (define FMT_SHIFT 16)
 (define FMT_MASK (ash 15 FMT_SHIFT))
 (define FMT_DEFAULT (ash 0 FMT_SHIFT))
@@ -860,65 +689,58 @@
 (define FMT_USER4 (ash 14 FMT_SHIFT))
 (define FMT_USER5 (ash 15 FMT_SHIFT))
 
-(define COLOR_SHIFT 20)
-(define COLOR_MASK (ash 7 COLOR_SHIFT))
-(define COLOR_BLACK (ash 0 COLOR_SHIFT))
-(define COLOR_RED (ash 1 COLOR_SHIFT))
-(define COLOR_GREEN (ash 2 COLOR_SHIFT))
-(define COLOR_BLUE (ash 3 COLOR_SHIFT))
-(define COLOR_YELLOW (ash 4 COLOR_SHIFT))
-(define COLOR_MAGENTA (ash 5 COLOR_SHIFT))
-(define COLOR_CYAN (ash 6 COLOR_SHIFT))
-(define COLOR_WHITE (ash 7 COLOR_SHIFT))
 
 (define HADJ_MASK (bit-or 4096 8192))
 (define HADJ_LEFT 0)
 (define HADJ_CENTER 8192)
 (define HADJ_RIGHT (bit-or 4096 8192))
 
-(define ITALIC 8)
-(define BOLD 16)
+(define (change-format buf r c name value)
+  (let* ((pos (make-position r c))
+	 (fmt (decode-format (get-format buf pos)))
+	 (field (assoc name fmt)))
+    (set-cdr! field value)
+    (set-format buf pos (encode-format fmt))))
 
-; Not needed, really
-;(define (name->font name)
-;  (cond ((equal? name "Times") TIMES)
-;	((equal? name "Courier") COURIER)
-;	((equal? name "Helvetica") HELVETICA)
-;	((equal? name "New Century Schoolbook") NEW_CENTURY)
-;	(t HELVETICA)))
+(define (line-format buf r c1 c2 name value)
+;  (set! c2 (min c2 (col-last-used r)))
+  (set! c2 (min c2 (max (col-last-used r) c1)))
+  (while (<= c1 c2)
+    (change-format buf r c1 name value)
+    (set! c1 (+ c1 1))))
 
-(define (change-font font mask)
-  (if (in-block (get-point) (get-blku) (get-blkl))
-    (change-format (position-row (get-blku))
-		   (position-row (get-blkl))
-		   (position-col (get-blku))
-		   (position-col (get-blkl))
-		   font
-		   mask)
-    (change-format (position-row (get-point))
-		   (position-row (get-point))
-		   (position-col (get-point))
-		   (position-col (get-point))
-		   font
-		   mask)))
+(define (area-format buf r1 c1 r2 c2 name value)
+;  (set! r2 (min r2 (line-last-used)))
+  (set! r2 (min r2 (max (line-last-used) r1)))
+  (while (<= r1 r2)
+    (line-format buf r1 c1 c2 name value)
+    (set! r1 (+ r1 1)))
+  (set-pr-scr))
 
-(define (toggle-fontflag flag)
-  (let ((oldfmt (get-format nil (get-point))))
-    (if (= (bit-and oldfmt flag) flag)
-      (change-font 0 flag)
-      (change-font flag flag))))
+; Change a single attribute
+(define (new-format name value)
+  (let ((r (position-row (get-point)))
+	(c (position-col (get-point)))
+	(r1 (position-row (get-blku)))
+	(c1 (position-col (get-blku)))
+	(r2 (position-row (get-blkl)))
+	(c2 (position-col (get-blkl))))
+    (if (and (>= r r1) (<= r r2) (>= c c1) (<= c c2))
+      (area-format nil r1 c1 r2 c2 name value)
+      (area-format nil r c r c name value))))
 
-(define (exec-siod)
-  (execute-interpreter-command 'SIOD))
+; Interactively select value from a list
+(define (new-attribute name)
+  (let ((value (select-attribute name)))
+    (if (not (null? value))
+      (new-format name value))))
 
-(define (exec-c)
-  (execute-interpreter-command 'C))
-
-(define (exec-guile)
-  (execute-interpreter-command 'Guile))
-
-(define (exec-tcl)
-  (execute-interpreter-command 'Tcl))
+; Used for bold, italic and underline.
+(define (toggle-format name)
+  (let* ((fmt (decode-format (get-format nil (get-point))))
+	 (field (assoc name fmt))
+	 (value (cdr field)))
+    (new-format name (if (eq? value 0) 1 0))))
 
 (define (edit-siod)
   (interpreter-test 'SIOD))
@@ -926,6 +748,9 @@
 (define (edit-guile)
   (interpreter-test 'Guile))
 
+(define (edit-python)
+  (interpreter-test 'Python))
+
 (define (edit-tcl)
   (interpreter-test 'Tcl))
 
@@ -952,37 +777,242 @@
 		  (cdr (assoc "Value" property)))))
 
 ; Load 1-2-3 compatibility functions
-(require (string-append SIAGHOME "/siag/123.scm"))
+(require (string-append datadir "/siag/123.scm"))
 
-; These are temporarily moved to .siagrc
-;(plugin-register "Oclock" "oclock" "/usr/local/siag/plugins/dummy")
-;(plugin-register "Xeyes" "xeyes" "/usr/local/siag/plugins/xeyes")
-;(plugin-register "Xload" "xload" "/usr/local/siag/plugins/xload")
-
-(require (string-append SIAGHOME "/plugins/plugin.scm"))
 
 (define (enter-date)
   (let* ((s (ask-for-str "Date:" ""))
-	 (t (strptime s "%x"))
-	 (b (number->string (mktime t) 10)))
-    (set-data nil b 0 ERROR (get-point))
-    (set-format nil (get-point) (+ FMT_DATE HELVETICA SIZE_10))
+	 (tim (strptime s "%x"))
+	 (p (get-point))
+	 (r (position-row p))
+	 (c (position-col p))
+	 (b (number->string (mktime tim) 10)))
+    (set-data nil b 0 ERROR p)
+    (change-format nil r c "style" "date")
     (calc-matrix nil)
     (set-pr-scr)
     (buffer-changed nil)))
 
 (define (enter-time)
   (let* ((s (ask-for-str "Time:" ""))
-	 (t (strptime s "%X"))
-	 (ti (append (list (assq 'sec t) (assq 'min t) (assq 'hour t))
+	 (tim (strptime s "%X"))
+	 (p (get-point))
+	 (r (position-row p))
+	 (c (position-col p))
+	 (ti (append (list (assq 'sec tim) (assq 'min tim) (assq 'hour tim))
 		     (cdddr (localtime 0))))
 	 (b (number->string (mktime ti) 10)))
-    (set-data nil b 0 ERROR (get-point))
-    (set-format nil (get-point) (+ FMT_TIME HELVETICA SIZE_10))
+    (set-data nil b 0 ERROR p)
+    (change-format nil r c "style" "time")
     (calc-matrix nil)
     (set-pr-scr)
     (buffer-changed nil)))
 
+; must check if already first because (- 0 1) would become -1 => last
+(define (move-sheet-up)
+  (let ((s (get-sheet)))
+    (if (> s 0)
+      (move-sheet s (- s 1)))))
 
-(require (string-append SIAGHOME "/siag/sort.scm"))
+; move-sheet checks new position, so we don't have to
+(define (move-sheet-down)
+  (let ((s (get-sheet)))
+    (move-sheet s (+ s 1))))
+
+; put first
+(define (move-sheet-top)
+  (let ((s (get-sheet)))
+    (move-sheet s 0)))
+
+; -1 means last
+(define (move-sheet-bottom)
+  (let ((s (get-sheet)))
+    (move-sheet s -1)))
+
+; Testing text plugin
+; To use, create docment with a single text plugin, then enter the
+; expression (text-plugin-test) into a cell
+; Includes the necessary error checking
+(define (text-plugin-test)
+  (let ((ph 0) (p nil) (q nil))
+    (plugin-write ph "EXEC !pwd\n")	; don't forget newline
+    (set! q (plugin-read ph))		; this should be 200 \.\n
+    (if (null? q)
+      "No response from plugin"
+      (if (substring-equal? "2" q 0 1)
+        (begin
+          (set! p (plugin-read ph))	; this should be current working dir
+  	  (if (null? p)
+	    "Empty reply from plugin"
+            (begin
+	      (set! q (plugin-read ph))	; this should be \.
+              (if (null? q)
+	        "Missing end marker from plugin"
+	        p))))
+	"Plugin command returned error"))))
+
+(require (string-append datadir "/siag/sort.scm"))
+
+; load the fonts
+(require (string-append datadir "/common/fonts.scm"))
+
+; and the styles
+(require (string-append datadir "/siag/styles.scm"))
+
+(define (change-margins buf)
+  (let* ((old (margin-get buf))
+	 (tm (car old))
+	 (bm (cadr old))
+	 (lm (caddr old))
+	 (rm (cadddr old))
+	 (hm (car (cddddr old)))
+	 (fm (cadr (cddddr old)))
+	 (new nil))
+    (form-begin)
+    (input-field "Top:" 80 "top" 100 tm)
+    (input-field "Bottom:" 80 "bottom" 100 bm)
+    (input-field "Left:" 80 "left" 100 lm)
+    (input-field "Right:" 80 "right" 100 rm)
+    (input-field "Header:" 80 "header" 100 hm)
+    (input-field "Footer:" 80 "footer" 100 fm)
+    (form-okbutton "Ok")
+    (form-cancelbutton "Cancel")
+    (set! new (form-end))
+    (set! lm (extract-number "left" new))
+    (set! rm (extract-number "right" new))
+    (set! tm (extract-number "top" new))
+    (set! bm (extract-number "bottom" new))
+    (set! hm (extract-number "header" new))
+    (set! fm (extract-number "footer" new))
+    (margin-set buf (list tm bm lm rm hm fm))))
+
+; Standard paper sizes
+
+(define paper-sizes
+	'(("Letter" 612 792)
+	  ("LetterSmall" 612 792)
+	  ("Tabloid" 792 1224)
+	  ("Ledger" 1224 792)
+	  ("Legal" 612 1008)
+	  ("Statement" 396 612)
+	  ("Executive" 540 720)
+	  ("A3" 842 1190)
+	  ("A4" 595 842)
+	  ("A4Small" 595 842)
+	  ("A5" 420 595)
+	  ("B4" 729 1032)
+	  ("B5" 516 729)
+	  ("Folio" 612 936)
+	  ("Quarto" 610 780)
+	  ("10x14" 720 1008)))
+
+(define (paper-add-size l)
+  (if l (begin (form-menuentry (caar l))
+	       (paper-add-size (cdr l)))))
+
+(define (change-paper-size buf)
+  (let* ((old (paper-get buf))
+	 (p (car old))
+	 (w (cadr old))
+	 (h (caddr old))
+	 (o (cadddr old))
+	 (pr (respect-prot-get buf))
+	 (new nil))
+    (form-begin)
+    (form-label "Size:")
+    (form-properties XtNwidth 80)
+    (form-menu "size")
+    (paper-add-size paper-sizes)
+    (form-properties XtNlabel p XtNwidth 100)
+    (form-newline)
+    (form-check "landscape" "Landscape")
+    (form-properties XtNstate o)
+    (form-newline)
+    (form-check "protect" "Respect protection")
+    (form-properties XtNstate pr)
+    (form-newline)
+    (form-okbutton "Ok")
+    (form-cancelbutton "Cancel")
+    (set! new (form-end))
+    (set! p (extract-string "size" new))
+    (set! w (cadr (assoc p paper-sizes)))
+    (set! h (caddr (assoc p paper-sizes)))
+    (set! o (extract-number "landscape" new))
+    (set! pr (extract-number "protect" new))
+    (paper-set buf (list p w h o))
+    (respect-prot-set "" pr)))
+
+(define (change-headfoot buf)
+  (let* ((old (headfoot-get buf))
+	  (h (car old))
+	  (f (cadr old))
+	  (fp (caddr old))
+	  (new nil))
+    (form-begin)
+    (input-field "Header:" 80 "header" 200 h)
+    (input-field "Footer:" 80 "footer" 200 f)
+    (form-check "first" "Header on first page")
+    (form-properties XtNstate fp)
+    (form-newline)
+    (form-okbutton "OK")
+    (form-cancelbutton "Cancel")
+    (set! new (form-end))
+    (set! h (extract-string "header" new))
+    (set! f (extract-string "footer" new))
+    (set! fp (extract-number "first" new))
+    (headfoot-set buf (list h f fp))))
+
+(define (change-first-page buf)
+  (let* ((old (first-page-get buf))
+	 (new nil))
+    (form-begin)
+    (input-field "First page number:" 160 "first" 100 old)
+    (form-okbutton "OK")
+    (form-cancelbutton "Cancel")
+    (set! new (form-end))
+    (first-page-set buf (extract-number "first" new))))
+
+; This is used by menu.scm and docs/menus.scm
+(define (add-pseudo-menu name) nil)
+
+
+; Return the total height (in pixels)
+; of all cells from r1 up to and including r2
+(define (totalheight r1 r2)
+  (let ((r r1)
+	(h 0))
+    (while (<= r r2)
+      (set! h (+ h (get-cell-height r)))
+      (set! r (+ r 1)))
+    h))
+
+; Return the total width (in pixels)
+; of all cells from c1 up to and including c2
+(define (totalwidth c1 c2)
+  (let ((c c1)
+	(w 0))
+    (while (<= c c2)
+      (set! w (+ w (get-cell-width c)))
+      (set! c (+ c 1)))
+    w))
+
+(define (try-executable dirs fn)
+  (cond ((null? dirs) nil)
+	((stat (string-append (car dirs) "/" fn)) t)
+	(t (try-executable (cdr dirs) fn))))
+
+(define (executable-file fn)
+  (let* ((path (getenv "PATH"))
+	 (dirs (strbreakup path ":")))
+    (try-executable dirs fn)))
+
+(define (register-converters)
+  (if (executable-file "xls2csv")
+    (register-converter "MS Excel (*.xls)"
+		        "*.csv" "xls2csv %s > %s"
+		        nil nil))
+  (if (executable-file "blaha")
+    (register-converter "Blaha (*.blaha)"
+			"*.txt" "blaha < %s > %s"
+			"*.txt" "blaha < %s > %s")))
 
