@@ -1,3 +1,23 @@
+/*
+   Siag, Scheme In A Grid
+   Copyright (C) 1996, 1997  Ulric Eriksson <ulric@edu.stockholm.se>
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+   MA 02111-1307, USA.
+ */
+
 /* user_interface.h
 
    These are all the functions that the user interface must implement
@@ -7,9 +27,8 @@
    This file is included from calc.h and from [xt]siag.h.
 */
 
-/*extern Window xwindow_of_window(window *);*/
+#include <stdio.h>
 
-extern void interp_startup(void);
 extern int select_file(char *, char *, char **, char *);
 extern int alert_box(char *, char **, int);
 extern int select_from_list(char *, char **, int);
@@ -19,22 +38,14 @@ extern void error_box(char *);
 extern char *ps_fontname(int);
 extern int ps_text_width(int, char *);
 extern int ps_font_descent(int);
-extern int ps_font_height(long);
+extern int ps_font_height(int);
 extern int ps_embed_print(FILE *, char *, int, int);
 extern int ps_font_size(int);
 
-/* from input.c (now window.h) */
+/* from window.c */
 extern int ask_for_str_comp(char *, char *, int (*)(char *));
 extern int ask_for_str(char *, char *);
 extern int edit_cell(char *, char *);
 extern int macro_flag;
 extern textbuf kbd_macro;
 
-/* plugin functions */
-extern int siag_plugin_start(char *);
-extern int siag_plugin_stop(int);
-extern int siag_plugin_save(int, char *);
-extern int siag_plugin_hide(int);
-extern int siag_plugin_print(int, FILE *);
-extern int siag_plugin_size_get(int, unsigned long *, unsigned long *);
-
