@@ -79,13 +79,18 @@ gjc@world.std.com
 #include "siod.h"
 #include "siodp.h"
 
+/* someone didn't get -DSIAG automatically; this is a workaround */
+#ifndef SIAG
+#define SIAG
+#endif
+
 #ifdef SIAG
 #include "silence.i"
 #endif	/* SIAG */
 
 static void init_slib_version(void)
 {setvar(cintern("*slib-version*"),
-	cintern("$Id: slib.c,v 1.7 1997/04/04 02:29:43 gjc Exp $"),
+	cintern("$Id: slib.c,v 1.2 1999/01/05 22:40:52 ulric Exp $"),
 	NIL);}
 
 char *siod_version(void)
