@@ -9,14 +9,23 @@
    convergence going on.
  */
 
-#ifdef sun	/* what a kludge /Ulric */
-#ifndef __EXTENSIONS__
-#define __EXTENSIONS__
-#endif
-#endif
+/* ---
+981205 Autoconf support by Ulric
 
-#ifndef _XOPEN_SOURCE	/* and another */
-#define _XOPEN_SOURCE
+Removed all "#ifdef unix" et al, because we only use this for Unix
+Removed all VMS specific code.
+Removed all WIN32 specific code.
+Most "#if defined(thissystem) || defined(thatsystem)" replaced with
+autoconf tests.
+Removed all dynamic library code.
+--- */
+
+/* start of Siag-specific attempts to make this module more portable */
+
+#include "../config.h"
+
+#ifndef unix
+#define unix
 #endif
 
 #include <stdio.h>
@@ -29,8 +38,8 @@
 #include <time.h>
 #include <errno.h>
 #include <stdarg.h>
+#include <limits.h>
 
-#if defined(unix)
 #include <unistd.h>
 #include <dirent.h>
 #include <sys/types.h>
@@ -42,64 +51,28 @@
 #include <grp.h>
 #include <utime.h>
 #include <sys/fcntl.h>
+
+
+#ifndef PATH_MAX
+#define PATH_MAX 1024
 #endif
 
-#if defined(__osf__) || defined(sun)
+#ifdef HAVE_SYS_MODE_H
 #include <sys/mode.h>
 #endif
-
-#if defined(__osf__) || defined(SUN5) || defined(linux)	/* Ulric */
+#ifdef HAVE_FNMATCH_H
 #include <fnmatch.h>
 #endif
-
-#if defined(__osf__)
-#include <rld_interface.h>
-#endif
-
-#if defined(hpux)
-#include <dl.h>
-#endif
-
-#ifndef SIAG
-#if defined(__osf__) || defined(sun) || defined(linux) || defined(sgi)
-#include <dlfcn.h>
-#endif
-#endif
-
-#if defined(sun)
+#ifdef HAVE_CRYPT_H
 #include <crypt.h>
-#include <limits.h>
+#endif
+#ifdef HAVE_SYS_MKDEV_H
 #include <sys/mkdev.h>
+#endif
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
 
-#if defined(sgi)
-#include <limits.h>
-#endif
-
-#if defined(hpux)
-#define PATH_MAX MAXPATHLEN
-#endif
-
-#if defined(VMS)
-#include <unixlib.h>
-#include <stat.h>
-#include <ssdef.h>
-#include <descrip.h>
-#include <lib$routines.h>
-#include <descrip.h>
-#include <ssdef.h>
-#include <iodef.h>
-#include <lnmdef.h>
-#endif
-
-#ifdef WIN32
-#include <windows.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <process.h>
-#include <direct.h>
-#endif
 
 #include "siod.h"
 #include "siodp.h"
@@ -107,7 +80,7 @@
 
 static void init_slibu_version(void)
 {setvar(cintern("*slibu-version*"),
-	cintern("$Id: slibu.c,v 1.12 1997/03/04 18:44:16 gjc Exp $"),
+	cintern("$Id: slibu.c,v 1.5 1999/01/31 00:05:52 ulric Exp $"),
 	NIL);}
 
 
@@ -128,15 +101,12 @@ LISP lsystem(LISP args)
  else
    return(flocons(retval));}
 
-#ifndef WIN32
 LISP lgetuid(void)
 {return(flocons(getuid()));}
 
 LISP lgetgid(void)
 {return(flocons(getgid()));}
-#endif
 
-#ifdef unix
 
 LISP lcrypt(LISP key,LISP salt)
 {char *result;
@@ -145,14 +115,6 @@ LISP lcrypt(LISP key,LISP salt)
  else
    return(NIL);}
 
-#endif
-
-#if defined(unix) || defined(WIN32)
-
-#if defined(WIN32)
-#define getcwd _getcwd
-#define PATH_MAX _MAX_PATH
-#endif
 
 LISP lgetcwd(void)
 {char path[PATH_MAX+1];
@@ -161,10 +123,6 @@ LISP lgetcwd(void)
  else
    return(err("getcwd",llast_c_errmsg(-1)));}
 
-#endif
-
-#ifdef unix
-
 
 LISP ldecode_pwent(struct passwd *p)
 {return(symalist(
@@ -174,6 +132,7 @@ LISP ldecode_pwent(struct passwd *p)
 		 "gid",flocons(p->pw_gid),
 		 "dir",strcons(strlen(p->pw_dir),p->pw_dir),
 		 "gecos",strcons(strlen(p->pw_gecos),p->pw_gecos),
+/* FIXME: this is horrible */
 #if defined(__osf__) || defined(hpux) || defined(sun)
 		 "comment",strcons(strlen(p->pw_comment),p->pw_comment),
 #endif
@@ -205,6 +164,7 @@ void lencode_pwent(LISP alist,struct passwd *p)
  p->pw_gid = longfield("gid",alist);
  p->pw_dir = strfield("dir",alist);
  p->pw_gecos = strfield("gecos",alist);
+/* FIXME: */
 #if defined(__osf__) || defined(hpux) || defined(sun)
  p->pw_comment = strfield("comment",alist);
 #endif
@@ -260,7 +220,7 @@ LISP lendpwent(void)
  no_interrupt(iflag);
  return(NIL);}
 
-#if defined(__osf__)
+#if defined(HAVE_SETPWFILE)
 LISP lsetpwfile(LISP fname)
 {int iflag = no_interrupt(1);
  setpwfile(get_c_string(fname));
@@ -268,6 +228,7 @@ LISP lsetpwfile(LISP fname)
  return(NIL);}
 #endif
 
+#ifdef HAVE_PUTPWENT
 LISP lputpwent(LISP alist,LISP file)
 {int iflag = no_interrupt(1);
  int status;
@@ -276,6 +237,7 @@ LISP lputpwent(LISP alist,LISP file)
  status = putpwent(&p,get_c_file(file,NULL));
  no_interrupt(iflag);
  return(NIL);}
+#endif
 
 LISP laccess_problem(LISP lfname,LISP lacc)
 {char *fname = get_c_string(lfname);
@@ -362,30 +324,20 @@ LISP lrename(LISP p1,LISP p2)
  no_interrupt(iflag);
  return(NIL);}
 
-#endif
+
+/* Changing all of this to rand so it conforms to C /Ulric */
 
 LISP lrandom(LISP n)
 {int res;
-#if defined(hpux) || defined(vms) || defined(sun) || defined(sgi) || defined(WIN32)
  res = rand();
-#endif
-#if defined(__osf__) || defined(linux)
- res = random();
-#endif
  return(flocons(NNULLP(n) ? res % get_c_long(n) : res));}
 
 LISP lsrandom(LISP n)
 {long seed;
  seed = get_c_long(n);
-#if defined(hpux) || defined(vms) || defined(sun) || defined(sgi) || defined(WIN32)
  srand(seed);
-#endif
-#if defined(__osf__) || defined(linux)
- srandom(seed);
-#endif
  return(NIL);}
 
-#ifdef unix
 
 LISP lfork(void)
 {int iflag;
@@ -400,7 +352,6 @@ LISP lfork(void)
  no_interrupt(iflag);
  return(flocons(pid));}
 
-#endif
 
 char **list2char(LISP *safe,LISP v)
 {char **x,*tmp;
@@ -414,7 +365,6 @@ char **list2char(LISP *safe,LISP v)
     strcpy(x[j],tmp);}
  return(x);}
 
-#ifdef unix
 
 LISP lexec(LISP path,LISP args,LISP env)
 {int iflag;
@@ -441,7 +391,6 @@ LISP lnice(LISP val)
  no_interrupt(iflag);
  return(flocons(n));}
 
-#endif
 
 int assemble_options(LISP l, ...)
 {int result = 0,val,noptions,nmask = 0;
@@ -466,7 +415,6 @@ int assemble_options(LISP l, ...)
    err("contains undefined options",l);
  return(result);}
 
-#ifdef unix
 
 LISP lwait(LISP lpid,LISP loptions)
 {pid_t pid,ret;
@@ -503,12 +451,10 @@ LISP lkill(LISP pid,LISP sig)
    no_interrupt(iflag);
  return(NIL);}
 
-#endif
 
 LISP lgetpid(void)
 {return(flocons(getpid()));}
 
-#ifdef unix
 LISP lgetpgrp(void)
 {return(flocons(getpgrp()));}
 
@@ -527,12 +473,9 @@ LISP lgetgrgid(LISP n)
  no_interrupt(iflag);
  return(result);}
 
-#endif
 
-#ifndef WIN32
 LISP lgetppid(void)
 {return(flocons(getppid()));}
-#endif
 
 LISP lmemref_byte(LISP addr)
 {unsigned char *ptr = (unsigned char *) get_c_long(addr);
@@ -550,7 +493,6 @@ LISP ltrunc(LISP x)
  i = (long) FLONM(x);
  return(flocons((double) i));}
 
-#ifdef unix
 LISP lputenv(LISP lstr)
 {char *orig,*cpy;
  orig = get_c_string(lstr);
@@ -562,7 +504,7 @@ LISP lputenv(LISP lstr)
    return(err("putenv",llast_c_errmsg(-1)));
  else
    return(NIL);}
-#endif
+
 
 /* I hate doing this, but it is apparently necessary /Ulric */
 /*#ifdef MD5*/
@@ -609,7 +551,7 @@ LISP md5_final(LISP ctx)
 /*#endif*/	/* MD5 */
 /* End of Ulric's butchery */
 
-#if defined(__osf__) || defined(sun)
+#if defined(HAVE_GETRLIMIT) && defined(RLIMIT_CPU)
 
 void handle_sigxcpu(int sig)
 {struct rlimit x;
@@ -642,7 +584,6 @@ LISP cpu_usage_limits(LISP soft,LISP hard)
 
 #endif
 
-#if defined(unix)
 
 static int handle_sigalrm_flag = 0;
 
@@ -667,10 +608,8 @@ LISP lalarm(LISP seconds,LISP flag)
  no_interrupt(iflag);
  return(flocons(retval));}
 
-#endif
 
-
-#if defined(__osf__) || defined(SUN5)
+#if defined(HAVE_GETRUSAGE) && defined(RUSAGE_SELF) && defined(RUSAGE_CHILDREN)
 
 #define TV_FRAC(x) (((double)x.tv_usec) * 1.0e-6)
 
@@ -711,7 +650,8 @@ LISP current_resource_usage(LISP kind)
 
 #endif
 
-#ifdef unix
+
+/* FIXME: there are standard autoconf macros to make this more portable */
 
 LISP l_opendir(LISP name)
 {long iflag;
@@ -759,6 +699,7 @@ LISP l_readdir(LISP v)
  r = readdir(d);
  no_interrupt(iflag);
  if (!r) return(NIL);
+/* FIXME: the following is unsatisfactory */
 #if defined(sun) || defined(sgi) || defined(linux)
  namlen = safe_strlen(r->d_name,r->d_reclen);
 #else
@@ -771,97 +712,6 @@ void opendir_prin1(LISP ptr,struct gen_printio *f)
  sprintf(buffer,"#<OPENDIR %p>",get_opendir(ptr,0));
  gput_st(f,buffer);}
 
-#endif
-
-#ifdef WIN32
-
-typedef struct
-{long count;
- HANDLE h;
- WIN32_FIND_DATA s;} DIR;
-
-LISP llast_win32_errmsg(DWORD status)
-{DWORD len,msgcode;
- char buffer[256];
- msgcode = (status == 0) ? GetLastError() : status;
- len = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM |
-	                 FORMAT_MESSAGE_IGNORE_INSERTS |
-					 FORMAT_MESSAGE_MAX_WIDTH_MASK,
-	                 0,
-					 msgcode,
-					 0, /* what language? */
-					 buffer,
-					 sizeof(buffer),
-					 NULL);
- if (len)
-   return(strcons(len,buffer));
- else
-   return(flocons(msgcode));}
-
-LISP l_opendir(LISP name)
-{long iflag;
- LISP value;
- DIR *d;
- iflag = no_interrupt(1);
- value = cons(NIL,NIL);
- d = (DIR *) must_malloc(sizeof(DIR));
- d->h = INVALID_HANDLE_VALUE;
- value->type = (short) tc_opendir;
- d->count = 0;
- CAR(value) = (LISP) d;
- if ((d->h = FindFirstFile(get_c_string(name),&d->s)) == INVALID_HANDLE_VALUE)
-   return(err("FindFirstFile",llast_win32_errmsg(0)));
- no_interrupt(iflag);
- return(value);}
-
-DIR *get_opendir(LISP v,long oflag)
-{if NTYPEP(v,tc_opendir) err("not an opendir",v);
- if NULLP(CAR(v))
-   {if (oflag) err("opendir not open",v);
-    return(NULL);}
- return((DIR *)CAR(v));}
-
-LISP l_closedir(LISP v)
-{long iflag;
- DIR *d;
- HANDLE h;
- iflag = no_interrupt(1);
- d = get_opendir(v,1);
- CAR(v) = NIL;
- h = d->h;
- free(d);
- if ((h != INVALID_HANDLE_VALUE) && !FindClose(h))
-   return(err("closedir",llast_win32_errmsg(0)));
- no_interrupt(iflag);
- return(NIL);}
-
-void  opendir_gc_free(LISP v)
-{DIR *d;
- if ((d = get_opendir(v,0)))
-   {FindClose(d->h);
-    free(d);
-	CAR(v) = NIL;}}
-
-LISP l_readdir(LISP v)
-{long iflag;
- DIR *d;
- d = get_opendir(v,1);
- iflag = no_interrupt(1);
- if (d->count > 0)
-   if (!FindNextFile(d->h,&d->s))
-	 if (GetLastError() == ERROR_NO_MORE_FILES)
-	   {no_interrupt(1);
-        return(NIL);}
- ++d->count;
- no_interrupt(iflag);
- return(strcons(-1,d->s.cFileName));}
-
-void opendir_prin1(LISP ptr,struct gen_printio *f)
-{char buffer[256];
- sprintf(buffer,"#<OPENDIR %p>",get_opendir(ptr,0));
- gput_st(f,buffer);}
-
-#endif
 
 LISP file_times(LISP fname)
 {struct stat st;
@@ -875,7 +725,6 @@ LISP file_times(LISP fname)
    return(cons(flocons(st.st_ctime),
 	       cons(flocons(st.st_mtime),NIL)));}
 
-#ifdef unix
 
 LISP decode_st_moden(mode_t mode)
 {LISP ret = NIL;
@@ -931,6 +780,7 @@ LISP decode_stat(struct stat *s)
 		 "ctime",flocons(s->st_ctime),
 		 "blksize",flocons(s->st_blksize),
 		 "blocks",flocons(s->st_blocks),
+/* FIXME: should check for these fields, not for the system */
 #if defined(__osf__)
 		 "flags",flocons(s->st_flags),
 		 "gen",flocons(s->st_gen),
@@ -966,7 +816,7 @@ LISP l_fstat(LISP f)
  else
    return(decode_stat(&st));}
 
-#if defined(__osf__) || defined(SUN5) || defined(linux)
+#ifdef HAVE_FNMATCH
 
 LISP l_fnmatch(LISP pat,LISP str,LISP flgs)
 {if (fnmatch(get_c_string(pat),
@@ -987,7 +837,7 @@ LISP lutime(LISP fname,LISP mod,LISP ac)
  else
    return(NIL);}
 
-LISP lchmod(LISP path,LISP mode)
+LISP l_chmod(LISP path,LISP mode)
 {if (chmod(get_c_string(path),get_c_long(mode)))
    return(err("chmod",llast_c_errmsg(-1)));
  else
@@ -1043,16 +893,11 @@ LISP lF_SETLKW(LISP fd,LISP ltype,LISP whence,LISP start,LISP len)
 LISP lF_GETLK(LISP fd,LISP ltype,LISP whence,LISP start,LISP len)
 {return(gsetlk(F_GETLK,fd,ltype,whence,start,len));}
 
-#endif
 
 LISP delete_file(LISP fname)
 {int iflag,ret;
  iflag = no_interrupt(1);
-#ifdef VMS
- ret = delete(get_c_string(fname));
-#else
- ret = unlink(get_c_string(fname));
-#endif
+ ret = remove(get_c_string(fname));
  no_interrupt(iflag);
  if (ret)
    return(strcons(-1,last_c_errmsg(-1)));
@@ -1072,99 +917,6 @@ LISP utime2str(LISP u)
  else
    return(NIL);}
 
-#ifdef WIN32
-LISP win32_debug(void)
-{DebugBreak();
- return(NIL);}
-#endif
-
-#ifdef VMS
-
-LISP vms_debug(arg)
-     LISP arg;
-{unsigned char arg1[257];
- char *data;
- if NULLP(arg)
-   lib$signal(SS$_DEBUG,0);
- else
-   {data = get_c_string(arg);
-    arg1[0] = strlen(data);
-    memcpy(&arg1[1],data,arg1[0]);
-    lib$signal(SS$_DEBUG,1,arg1);}
- return(NIL);}
-
-struct dsc$descriptor *set_dsc_cst(struct dsc$descriptor *d,char *s)
-{d->dsc$w_length = strlen(s);
- d->dsc$b_dtype = DSC$K_DTYPE_T;
- d->dsc$b_class = DSC$K_CLASS_S;
- d->dsc$a_pointer = s;
- return(d);}
-
-
-void err_vms(long retval)
-{char *errmsg,buff[100];
- if (errmsg = strerror(EVMSERR,retval))
-   err(errmsg,NIL);
- else
-   {sprintf(buff,"VMS ERROR %d",retval);
-    err(buff,NIL);}}
-
-LISP lcrembx(LISP l)
-{LISP tmp;
- short chan;
- int prmflg,maxmsg,bufquo,promsk,acmode,iflag,retval;
- struct dsc$descriptor lognam;
- set_dsc_cst(&lognam,get_c_string(car(l)));
- tmp = cadr(assq(cintern("prmflg"),l));
- prmflg = NNULLP(tmp) ? 1 : 0;
- tmp = cadr(assq(cintern("maxmsg"),l));
- maxmsg = NNULLP(tmp) ? get_c_long(tmp) : 0;
- tmp = cadr(assq(cintern("bufquo"),l));
- bufquo = NNULLP(tmp) ? get_c_long(tmp) : 0;
- tmp = cadr(assq(cintern("promsk"),l));
- promsk = NNULLP(tmp) ? get_c_long(tmp) : 0;
- tmp = cadr(assq(cintern("acmode"),l));
- acmode = NNULLP(tmp) ? get_c_long(tmp) : 0;
- tmp = cons(flocons(-1),leval(sym_channels,NIL));
- iflag = no_interrupt(1);
- retval = sys$crembx(prmflg,&chan,maxmsg,bufquo,promsk,acmode,&lognam);
- if (retval != SS$_NORMAL)
-   {no_interrupt(iflag);
-    err_vms(retval);}
- setvar(sym_channels,tmp,NIL);
- tmp = car(tmp);
- tmp->storage_as.flonum.data = chan;
- no_interrupt(iflag);
- return(tmp);}
-
-LISP lset_logical(LISP name,LISP value,LISP table,LISP attributes)
-{struct dsc$descriptor dname,dvalue,dtable;
- long status,iflag;
- iflag = no_interrupt(1);
- status = lib$set_logical(set_dsc_cst(&dname,get_c_string(name)),
-			  NULLP(value) ? 0 : set_dsc_cst(&dvalue,
-							 get_c_string(value)),
-			  NULLP(table) ? 0 : set_dsc_cst(&dtable,
-							 get_c_string(table)),
-			  assemble_options(attributes,
-					   "NO_ALIAS",LNM$M_NO_ALIAS,
-					   "CONFINE",LNM$M_CONFINE,
-					   "CRELOG",LNM$M_CRELOG,
-					   "TABLE",LNM$M_TABLE,
-					   "CONCEALED",LNM$M_CONCEALED,
-					   "TERMINAL",LNM$M_TERMINAL,
-					   "EXISTS",LNM$M_EXISTS,
-					   "SHAREABLE",LNM$M_SHAREABLE,
-					   "CREATE_IF",LNM$M_CREATE_IF,
-					   "CASE_BLIND",LNM$M_CASE_BLIND,
-					   NULL),
-			  0);
- if (status != SS$_NORMAL)
-   err_vms(status);
- no_interrupt(iflag);
- return(NIL);}
-
-#endif
 
 LISP lgetenv(LISP var)
 {char *str;
@@ -1210,6 +962,7 @@ LISP http_date(LISP value)
    t->tm_sec));
  return(strcons(strlen(buff),buff));}
 
+/* FIXME: what makes this osf-only? */
 #if defined(__osf__)
 
 LISP http_date_parse(LISP input)
@@ -1250,34 +1003,12 @@ LISP http_date_parse(LISP input)
 #endif
 
 
-#ifdef hpux
-long usleep(unsigned int winks)	/* added, dcd */
-{
-  struct timeval sleepytime;
-  sleepytime.tv_sec = winks / 1000000;
-  sleepytime.tv_usec = winks % 1000000;
-  return select(0,0,0,0,&sleepytime);
-}
-#endif
-
-#if /* Ulric commented out: defined(sun) || */ defined(sgi)
-long usleep(unsigned int winks)
-{struct timespec x;
- x.tv_sec = winks / 1000000;
- x.tv_nsec = (winks % 1000000) * 1000;
- return(nanosleep(&x,NULL));}
-#endif
-
 LISP lsleep(LISP ns)
 {double val = get_c_double(ns);
-#ifdef unix
+#ifdef HAVE_USLEEP
  usleep((unsigned int)(val * 1.0e6));
-#else
-#ifdef WIN32
- Sleep((DWORD)(val * 1000));
 #else
  sleep((unsigned int)val);
-#endif
 #endif
  return(NIL);}
 
@@ -1399,7 +1130,7 @@ LISP lgets(LISP file,LISP buffn)
  no_interrupt(iflag);
  return(NIL);}
 
-LISP readline(LISP file)
+LISP lreadline(LISP file)
 {LISP result;
  char *start,*ptr;
  result = lgets(file,NIL);
@@ -1416,7 +1147,6 @@ LISP readline(LISP file)
       and then append the results */
    return(result);}
 
-#ifndef WIN32
 
 LISP l_chown(LISP path,LISP uid,LISP gid)
 {long iflag;
@@ -1426,10 +1156,8 @@ LISP l_chown(LISP path,LISP uid,LISP gid)
  no_interrupt(iflag);
  return(NIL);}
 
-#endif
 
-/* Ulric was here: added hpux */
-#if defined(unix) && !defined(linux) && !defined(hpux)
+#ifdef HAVE_LCHOWN
 LISP l_lchown(LISP path,LISP uid,LISP gid)
 {long iflag;
  iflag = no_interrupt(1);
@@ -1440,8 +1168,6 @@ LISP l_lchown(LISP path,LISP uid,LISP gid)
 #endif
 
 
-#ifdef unix
-
 LISP popen_l(LISP name,LISP how)
 {return(fopen_cg(popen,
 		 get_c_string(name),
@@ -1467,130 +1193,7 @@ LISP pclose_l(LISP ptr)
    err("pclose",llast_c_errmsg(xerrno));
  return(flocons(retval));}
 
-#endif
 
-LISP so_init_name(LISP fname,LISP iname)
-{LISP init_name;
- if NNULLP(iname)
-   init_name = iname;
- else
-   {init_name = car(last(lstrbreakup(fname,cintern("/"))));
-#if !defined(VMS)
-    init_name = lstrunbreakup(butlast(lstrbreakup(init_name,cintern("."))),
-			      cintern("."));
-#endif
-    init_name = string_append(listn(2,cintern("init_"),init_name));}
- return(intern(init_name));}
-
-LISP so_ext(LISP fname)
-{char *ext = ".so";
- LISP lext;
-#if defined(hpux)
- ext = ".sl";
-#endif
-#if defined(vms)
- ext = "";
-#endif
-#if defined(WIN32)
- ext = ".dll";
-#endif
- lext = strcons(strlen(ext),ext);
- if NULLP(fname)
-   return(lext);
- else
-   return(string_append(listn(2,fname,lext)));}
-
-#ifndef SIAG
-
-LISP load_so(LISP fname,LISP iname)
-     /* note: error cases can leak memory in this procedure. */
-{LISP init_name;
- void (*fcn)(void) = NULL;
-#if defined(__osf__) || defined(sun) || defined(linux) || defined(sgi)
- void *handle;
-#endif
-#if defined(hpux)
- shl_t handle;
-#endif
-#if defined(VMS)
- struct dsc$descriptor filename,symbol,defaultd;
- long status;
- LISP dsym;
-#endif
-#ifdef WIN32
- HINSTANCE handle;
-#endif
- long iflag;
- init_name = so_init_name(fname,iname);
- iflag = no_interrupt(1);
- if (siod_verbose_check(3))
-   {put_st("so-loading ");
-    put_st(get_c_string(fname));
-    put_st("\n");}
-#if defined(__osf__) || defined(sun) || defined(linux) || defined(sgi)
-#if !defined(__osf__)
- /* Observed bug: values of LD_LIBRARY_PATH established with putenv
-    -after- a process has started are ignored. Work around follows. */
- if (access(get_c_string(fname),F_OK))
-   fname = string_append(listn(3,
-			       strcons(-1,siod_lib),
-			       strcons(-1,"/"),
-			       fname));
-#endif
-#if 0	/* Ulric was here */
- if (!(handle = dlopen(get_c_string(fname),RTLD_LAZY)))
-   err(dlerror(),fname);
- if (!(fcn = dlsym(handle,get_c_string(init_name))))
-   err(dlerror(),init_name);
-#endif
-#endif
-#if defined(hpux)
- if (!(handle = shl_load(get_c_string(fname),BIND_DEFERRED,0L)))
-   err("shl_load",llast_c_errmsg(errno));
- if (shl_findsym(&handle,get_c_string(init_name),TYPE_PROCEDURE,&fcn))
-   err("shl_findsym",llast_c_errmsg(errno));
-#endif
-#if defined(VMS)
- dsym = cintern("*require-so-dir*");
- if (NNULLP(symbol_boundp(dsym,NIL)) && NNULLP(symbol_value(dsym,NIL)))
-   set_dsc_cst(&defaultd,get_c_string(symbol_value(dsym,NIL)));
- else
-   dsym = NIL;
- status = lib$find_image_symbol(set_dsc_cst(&filename,
-					    get_c_string(fname)),
-				set_dsc_cst(&symbol,
-					    get_c_string(init_name)),
-				&fcn,
-				NULLP(dsym) ? 0 : &defaultd);
- if (status != SS$_NORMAL)
-   err_vms(status);
-#endif
-#ifdef WIN32
- if (!(handle = LoadLibrary(get_c_string(fname))))
-   err("LoadLibrary",fname);
- if (!(fcn = (LPVOID)GetProcAddress(handle,get_c_string(init_name))))
-   err("GetProcAddress",init_name);
-#endif
- if (fcn)
-   (*fcn)();
- else
-   err("did not load function",init_name);
- no_interrupt(iflag);
- if (siod_verbose_check(3))
-   put_st("done.\n");
- return(init_name);}
-
-LISP require_so(LISP fname)
-{LISP init_name;
- init_name = so_init_name(fname,NIL);
- if (NULLP(symbol_boundp(init_name,NIL)) ||
-     NULLP(symbol_value(init_name,NIL)))
-   {load_so(fname,NIL);
-    return(setvar(init_name,a_true_value(),NIL));}
- else
-   return(NIL);}
-
-#endif	/* SIAG */
 
 LISP siod_lib_l(void)
 {return(rintern(siod_lib));}
@@ -1624,6 +1227,7 @@ LISP decode_tm(struct tm *t)
 		 "wday",flocons(t->tm_wday),
 		 "yday",flocons(t->tm_yday),
 		 "isdst",flocons(t->tm_isdst),
+/* FIXME: check for record fields, not system */
 #if defined(__osf__)
 		 "gmtoff",flocons(t->__tm_gmtoff),
 		 "tm_zone",(t->__tm_zone) ? rintern(t->__tm_zone) : NIL,
@@ -1696,11 +1300,9 @@ LISP lgmtime(LISP value)
  else
    return(err("gmtime",llast_c_errmsg(-1)));}
 
-#if defined(unix)
 LISP ltzset(void)
 {tzset();
  return(NIL);}
-#endif
 
 LISP lmktime(LISP alist)
 {struct tm tm;
@@ -1709,7 +1311,7 @@ LISP lmktime(LISP alist)
  t = mktime(&tm);
  return(flocons(t));}
 
-#if defined(__osf__) || defined(SUN5) || defined(SIAG)
+#if defined(HAVE_STRPTIME)
 
 LISP lstrptime(LISP str,LISP fmt,LISP in)
 {struct tm tm;
@@ -1721,7 +1323,6 @@ LISP lstrptime(LISP str,LISP fmt,LISP in)
 
 #endif
 
-#ifdef unix
 
 LISP lstrftime(LISP fmt,LISP in)
 {struct tm tm;
@@ -1741,27 +1342,20 @@ LISP lstrftime(LISP fmt,LISP in)
  else
    return(NIL);}
 
-#endif
 
 LISP lchdir(LISP dir)
 {long iflag;
-#ifdef unix
  FILE *f;
  int fd;
-#endif
  char *path;
  switch(TYPE(dir))
    {case tc_c_file:
-#ifdef unix
       f = get_c_file(dir,NULL);
       fd = fileno(f);
       iflag = no_interrupt(1);
       if (fchdir(fd))
 	return(err("fchdir",llast_c_errmsg(-1)));
       no_interrupt(iflag);
-#else
-      err("fchdir not supported in os",NIL);
-#endif
       return(NIL);
     default:
       path = get_c_string(dir);
@@ -1771,17 +1365,7 @@ LISP lchdir(LISP dir)
       no_interrupt(iflag);
       return(NIL);}}
 
-#if defined(__osf__)
-LISP rld_pathnames(void)
-     /* this is a quick diagnostic to know what images we are running */
-{char *path;
- LISP result = NIL;
- for(path=_rld_first_pathname();path;path=_rld_next_pathname())
-   result = cons(strcons(strlen(path),path),result);
- return(nreverse(result));}
-#endif
 
-#ifdef unix
 LISP lgetpass(LISP lprompt)
 {long iflag;
  char *result;
@@ -1792,9 +1376,7 @@ LISP lgetpass(LISP lprompt)
    return(strcons(strlen(result),result));
  else
    return(NIL);}
-#endif
 
-#ifdef unix
 LISP lpipe(void)
 {int filedes[2];
  long iflag;
@@ -1811,7 +1393,6 @@ LISP lpipe(void)
     return(listn(2,f1,f2));}
  else
    return(err("pipe",llast_c_errmsg(-1)));}
-#endif
 
 #define CTYPE_FLOAT   1
 #define CTYPE_DOUBLE  2
@@ -1904,10 +1485,7 @@ LISP datlength(LISP dat,LISP ctype)
 
 void init_slibu(void)
 {long j;
-#if defined(unix)
  char *tmp1,*tmp2;
-#endif
-#if defined(unix) || defined(WIN32)
  tc_opendir = allocate_user_tc();
  set_gc_hooks(tc_opendir,
 	      NULL,
@@ -1916,47 +1494,33 @@ void init_slibu(void)
 	      opendir_gc_free,
 	      &j);
  set_print_hooks(tc_opendir,opendir_prin1);
-#endif
 
  gc_protect_sym(&sym_channels,"*channels*");
  setvar(sym_channels,NIL,NIL);
-#ifdef WIN32
- init_subr_0("win32-debug",win32_debug);
-#endif
-#ifdef VMS
- init_subr_1("vms-debug",vms_debug);
- init_lsubr("sys$crembx",lcrembx);
- init_subr_4("lib$set_logical",lset_logical);
-#endif
  init_lsubr("system",lsystem);
-#ifndef WIN32
  init_subr_0("getgid",lgetgid);
  init_subr_0("getuid",lgetuid);
-#endif
-#if defined(unix) || defined(WIN32)
  init_subr_0("getcwd",lgetcwd);
-#endif
-#ifdef unix
  init_subr_2("crypt",lcrypt);
  init_subr_1("getpwuid",lgetpwuid);
  init_subr_1("getpwnam",lgetpwnam);
  init_subr_0("getpwent",lgetpwent);
  init_subr_0("setpwent",lsetpwent);
  init_subr_0("endpwent",lendpwent);
-#if defined(__osf__)
+#if defined(HAVE_SETPWFILE)
  init_subr_1("setpwfile",lsetpwfile);
 #endif
+#if defined(HAVE_PUTPWENT)
  init_subr_2("putpwent",lputpwent);
+#endif
  init_subr_2("access-problem?",laccess_problem);
  init_subr_3("utime",lutime);
- init_subr_2("chmod",lchmod);
+ init_subr_2("chmod",l_chmod);
  init_subr_2("fchmod",lfchmod);
-#endif
  init_subr_1("random",lrandom);
  init_subr_1("srandom",lsrandom);
  init_subr_1("first",car);
  init_subr_1("rest",cdr);
-#ifdef unix
  init_subr_0("fork",lfork);
  init_subr_3("exec",lexec);
  init_subr_1("nice",lnice);
@@ -1964,33 +1528,26 @@ void init_slibu(void)
  init_subr_0("getpgrp",lgetpgrp);
  init_subr_1("getgrgid",lgetgrgid);
  init_subr_2("kill",lkill);
-#endif
  init_subr_1("%%%memref",lmemref_byte);
  init_subr_0("getpid",lgetpid);
-#ifndef WIN32
  init_subr_0("getppid",lgetppid);
-#endif
  init_subr_1("exit",lexit);
  init_subr_1("trunc",ltrunc);
-#ifdef unix
  init_subr_1("putenv",lputenv);
-#endif
 /*#ifdef MD5*/ /* butchery by Ulric */
  init_subr_0("md5-init",md5_init);
  init_subr_3("md5-update",md5_update);
  init_subr_1("md5-final",md5_final);
 /*#endif*/ /* MD5 */
-#if defined(__osf__) || defined(sun)
+#if defined(HAVE_GETRLIMIT) && defined(RLIMIT_CPU)
  init_subr_2("cpu-usage-limits",cpu_usage_limits);
 #endif
-#if defined(__osf__) || defined(SUN5)
+#if defined(HAVE_GETRUSAGE) && defined(RUSAGE_SELF) && defined(RUSAGE_CHILDREN)
  init_subr_1("current-resource-usage",current_resource_usage);
 #endif
-#if  defined(unix) || defined(WIN32)
  init_subr_1("opendir",l_opendir);
  init_subr_1("closedir",l_closedir);
  init_subr_1("readdir",l_readdir);
-#endif
  init_subr_1("delete-file",delete_file);
  init_subr_1("file-times",file_times);
  init_subr_1("unix-time->strtime",utime2str);
@@ -2001,21 +1558,18 @@ void init_slibu(void)
  init_subr_1("url-encode",url_encode);
  init_subr_1("url-decode",url_decode);
  init_subr_2("gets",lgets);
- init_subr_1("readline",readline);
+ init_subr_1("readline",lreadline);
  init_subr_1("html-encode",html_encode);
  init_subr_1("html-decode",html_decode);
-#ifdef unix
  init_subr_1("decode-file-mode",decode_st_mode);
  init_subr_1("encode-file-mode",encode_st_mode);
  init_subr_1("encode-open-flags",encode_open_flags);
  init_subr_1("stat",l_stat);
  init_subr_1("lstat",l_lstat);
  init_subr_1("fstat",l_fstat);
-#endif
-#if defined(__osf__) || defined(SUN5) || defined(linux)
+#if defined(HAVE_FNMATCH)
  init_subr_3("fnmatch",l_fnmatch);
 #endif
-#ifdef unix
  init_subr_2("symlink",lsymlink);
  init_subr_2("link",llink);
  init_subr_1("unlink",lunlink);
@@ -2023,32 +1577,18 @@ void init_slibu(void)
  init_subr_2("mkdir",lmkdir);
  init_subr_2("rename",lrename);
  init_subr_1("readlink",lreadlink);
-#endif
-#ifndef WIN32
  init_subr_3("chown",l_chown);
-#endif
-/* Ulric was here: added hpux */
-#if defined(unix) && !defined(linux) && !defined(hpux)
+#if defined(HAVE_LCHOWN)
  init_subr_3("lchown",l_lchown);
 #endif
  init_subr_1("http-date",http_date);
 #if defined(__osf__)
  init_subr_1("http-date-parse",http_date_parse);
 #endif
-#ifdef unix
  init_subr_2("popen",popen_l);
  init_subr_1("pclose",pclose_l);
-#endif
 
-#ifndef SIAG
 
- init_subr_2("load-so",load_so);
- init_subr_1("require-so",require_so);
- init_subr_1("so-ext",so_ext);
-
-#endif	/* SIAG */
-
-#ifdef unix
  setvar(cintern("SEEK_SET"),flocons(SEEK_SET),NIL);
  setvar(cintern("SEEK_CUR"),flocons(SEEK_CUR),NIL);
  setvar(cintern("SEEK_END"),flocons(SEEK_END),NIL);
@@ -2059,10 +1599,8 @@ void init_slibu(void)
  init_subr_5("F_SETLKW",lF_SETLKW);
  init_subr_5("F_GETLK",lF_GETLK);
 
-#endif
  init_subr_0("siod-lib",siod_lib_l);
 
-#ifdef unix
  if ((!(tmp1 = getenv(ld_library_path_env))) ||
      (!strstr(tmp1,siod_lib)))
    {tmp2 = (char *) must_malloc(strlen(ld_library_path_env) + 1 +
@@ -2075,33 +1613,18 @@ void init_slibu(void)
 	    siod_lib);
     /* note that we cannot free the string afterwards. */
     putenv(tmp2);}
-#endif
-#ifdef vms
- setvar(cintern("*require-so-dir*"),
-	string_append(listn(2,
-			    strcons(-1,siod_lib),
-			    strcons(-1,".EXE"))),
-	NIL);
-#endif
  init_subr_1("localtime",llocaltime);
  init_subr_1("gmtime",lgmtime);
-#if defined(unix)
  init_subr_0("tzset",ltzset);
-#endif
  init_subr_1("mktime",lmktime);
  init_subr_1("chdir",lchdir);
-#if defined(__osf__)
- init_subr_0("rld-pathnames",rld_pathnames);
-#endif
-#if defined(__osf__) || defined(SUN5) || defined(SIAG)
+#if defined(HAVE_STRPTIME)
  init_subr_3("strptime",lstrptime);
 #endif
-#ifdef unix
  init_subr_2("strftime",lstrftime);
  init_subr_1("getpass",lgetpass);
  init_subr_0("pipe",lpipe);
  init_subr_2("alarm",lalarm);
-#endif
 
  setvar(cintern("CTYPE_FLOAT"),flocons(CTYPE_FLOAT),NIL);
  setvar(cintern("CTYPE_DOUBLE"),flocons(CTYPE_DOUBLE),NIL);
