@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 1998  Ulric Eriksson <ulric@edu.stockholm.se>
+   Copyright (C) 1998-1999  Ulric Eriksson <ulric@edu.stockholm.se>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -26,7 +26,8 @@
 #include <X11/StringDefs.h>
 
 #include "../common/cmalloc.h"
-#include "xfonts.h"	/* from common */
+#include "../common/fonts.h"
+#include "xfonts.h"
 #include <X11/xpm.h>
 #include "embed.h"
 
@@ -35,7 +36,8 @@
 
 #include "RichtextP.h"
 
-static void plugin_coords(Widget, XtPointer, long *, long *);
+static void plugin_coords(Widget, XtPointer, int *, int *);
+static Boolean default_bop(XtPointer, int);
 
 #define offset(field) XtOffsetOf(RichtextRec, richtext.field)
 static XtResource resources[] = {
@@ -127,6 +129,14 @@ static XtResource resources[] = {
 		offset(style),
 		XtRImmediate,
 		(XtPointer)NULL
+	}, {
+		XtNrichtextBop,
+		XtCRichtextBop,
+		XtRPointer,
+		sizeof(XtPointer),
+		offset(bop),
+		XtRImmediate,
+		(XtPointer)default_bop
 	}, {
 		XtNrichtextText,
 		XtCRichtextText,
@@ -183,6 +193,14 @@ static XtResource resources[] = {
 		offset(right_margin),
 		XtRImmediate,
 		(XtPointer)72
+	}, {
+		XtNrichtextTabDistance,
+		XtCRichtextTabDistance,
+		XtRInt,
+		sizeof(int),
+		offset(tab_distance),
+		XtRImmediate,
+		(XtPointer)36
 	}, {
 		XtNrichtextPluginCoords,
 		XtCRichtextPluginCoords,
@@ -205,7 +223,13 @@ static void Realize(Widget, XtValueMask *, XSetWindowAttributes *);
 static void Destroy(Widget);
 static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal *);
 
-static void plugin_coords(Widget w, XtPointer p, long *x, long *y)
+/* by default, do not display Beginning Of Paragraph marks */
+static Boolean default_bop(XtPointer p, int i)
+{
+	return False;
+}
+
+static void plugin_coords(Widget w, XtPointer p, int *x, int *y)
 {
 	*x = *y = 0;
 }
@@ -215,12 +239,12 @@ static void RichtextAction(Widget, XEvent *, String *, Cardinal *);
 
 static XtActionsRec actions[] =
 {
-	{"richtext", RichtextAction},
+	{"richtext-action", RichtextAction},
 };
 
 /* translations */
 static char translations[] =
-"<Key>:		richtext()\n";
+"<Key>:		self-insert-char()\n";
 
 RichtextClassRec richtextClassRec = {
   { /* core fields */
@@ -298,6 +322,10 @@ static void Realize(Widget w, XtValueMask *valueMask,
 	RichtextWidget rtw = (RichtextWidget)w;
 	unsigned long fg, bg, blockbg;
 	XColor screen_color, exact_color;
+	Display *dpy;
+	char *name, *class;
+	XIMStyles *styles;
+	int i;
 
 	(*superclass->core_class.realize)(w, valueMask, attributes);
 	fg = BlackPixelOfScreen(XtScreen(w));
@@ -308,11 +336,30 @@ static void Realize(Widget w, XtValueMask *valueMask,
 	blockbg = screen_color.pixel;
 	rtw->richtext.clear_gc = get_gc(w, bg, fg, -1);
 	rtw->richtext.cell_gc = get_gc(w, fg, blockbg,
-			get_font(XtDisplay(w), CELL_FONT));
+			-1/*get_font(0)*/);
 	rtw->richtext.cursor_gc = get_gc(w, fg^bg, 0, -1);
 	XSetFunction(XtDisplay(w), rtw->richtext.cursor_gc, GXxor);
         XSetLineAttributes(XtDisplay(w), rtw->richtext.cursor_gc,
 			1, LineSolid, CapButt, JoinMiter);
+
+#ifdef HAVE_XCREATEIC
+	/* Set up input methods */
+	dpy = XtDisplay(w);
+	XtGetApplicationNameAndClass(dpy, &name, &class);
+	rtw->richtext.xim = XOpenIM(dpy, XtDatabase(dpy), name, class);
+	XGetIMValues(rtw->richtext.xim,
+		XNQueryInputStyle, &styles,
+		(char *)0);
+	for (i = 0; i < styles->count_styles; i++) {
+		if (styles->supported_styles[i] ==
+		    (XIMPreeditNothing | XIMStatusNothing)) break;
+	}
+	if (i == styles->count_styles) i = 0;
+	rtw->richtext.xic = XCreateIC(rtw->richtext.xim,
+		XNInputStyle, styles->supported_styles[i],
+		XNClientWindow, XtWindow(w),
+		(char *)0);
+#endif
 }
 
 static void Destroy(Widget w)
@@ -322,6 +369,10 @@ static void Destroy(Widget w)
 	XFreeGC(XtDisplay(w), rtw->richtext.clear_gc);
 	XFreeGC(XtDisplay(w), rtw->richtext.cell_gc);
 	XFreeGC(XtDisplay(w), rtw->richtext.cursor_gc);
+#ifdef HAVE_XCREATEIC
+	XDestroyIC(rtw->richtext.xic);
+	XCloseIM(rtw->richtext.xim);
+#endif
 }
 
 static Dimension row_height(RichtextWidget rtw, int row)
@@ -331,11 +382,14 @@ static Dimension row_height(RichtextWidget rtw, int row)
 	return 20;
 }
 
-/* forget about x for now */
+/* ---
+forget about x for now
+*/
+
 void richtext_global_coords(RichtextWidget tw,
-		long row, long col, long *x, long *y)
+		int row, int col, int *x, int *y)
 {
-	long i;
+	int i;
 
 	*x = *y = 0;
 
@@ -343,7 +397,11 @@ void richtext_global_coords(RichtextWidget tw,
 		*y += row_height(tw, i);
 }
 
-int inblock(RichtextWidget rtw, int r, int c)
+/* ---
+Check if we are in the block.
+*/
+
+static int inblock(RichtextWidget rtw, int r, int c)
 {
 	return ((r > rtw->richtext.sel_top_row ||
 		 (r == rtw->richtext.sel_top_row &&
@@ -353,21 +411,21 @@ int inblock(RichtextWidget rtw, int r, int c)
 		   c <= rtw->richtext.sel_bottom_col)));
 }
 
-static int ret_hadj(RichtextWidget rtw, long row)
+static int ret_hadj(RichtextWidget rtw, int row)
 {
 	if (rtw->richtext.adj_horiz)
 		return (*rtw->richtext.adj_horiz)(rtw->richtext.data, row);
 	return HADJ_LEFT;
 }
 
-static int ret_style(RichtextWidget rtw, long row)
+static int ret_style(RichtextWidget rtw, int row)
 {
 	if (rtw->richtext.style)
 		return (*rtw->richtext.style)(rtw->richtext.data, row);
 	return STY_DEFAULT;
 }
 
-static rich_char *ret_text(RichtextWidget rtw, long row)
+static rich_char *ret_text(RichtextWidget rtw, int row)
 {
 	if (rtw->richtext.text)
 		return (*rtw->richtext.text)(rtw->richtext.data, row);
@@ -380,7 +438,7 @@ static int line_width(RichtextWidget rtw, int row, int col)
         char *p;
         int i, l;
         unsigned int width;
-        XFontStruct *font_info;
+
 	line = ret_text(rtw, row);
 	if (line == NULL) return 0;
 
@@ -397,11 +455,10 @@ static int line_width(RichtextWidget rtw, int row, int col)
         width = 0;      /* accumulated width */
         for (i = 0; i < col && line[i].c; i++) {
                 if (line[i].c == '\t') {
-                        width += 36;
-                        width -= (width % 36);
+                        width += rtw->richtext.tab_distance;
+                        width -= (width % rtw->richtext.tab_distance);
                 } else {
-                        font_info = font_struct(XtDisplay(rtw), line[i].fmt & 127);
-                        width += XTextWidth(font_info, p+i, 1);
+			width += rc_width(line[i]);
                 }
         }
         cfree(p);
@@ -434,6 +491,8 @@ static int line_start(RichtextWidget rtw, int row)
         return x+rtw->richtext.left_margin;
 }
 
+/* ---
+*/
 void richtext_char2coords(RichtextWidget rtw,
 		int cell_row, int cell_col,
 		int *cell_x, int *cell_y)
@@ -446,6 +505,8 @@ void richtext_char2coords(RichtextWidget rtw,
                         +line_width(rtw, cell_row, cell_col);
 }
 
+/* ---
+*/
 void richtext_coords2char(RichtextWidget rtw,
 		int *cur_row, int *cur_col,
 		int cur_x, int cur_y)
@@ -515,14 +576,14 @@ static void toggle_cursor(RichtextWidget rtw)
 static void draw_line(RichtextWidget rtw, Drawable cell_win,
 		int y_base, int row, int clr)
 {
-        int font_index, color_index;
+	int color_index;
         int i;
-        Font font;
         unsigned long color;
         int height = row_height(rtw, row);
         int x_base = 0, old_x_base;
 	Display *display = XtDisplay(rtw);
         rich_char *line = ret_text(rtw, row);
+	int bop;
 
         if (clr) {
                 /* the 4 is from trial and error; should be from font metrics */
@@ -540,46 +601,60 @@ static void draw_line(RichtextWidget rtw, Drawable cell_win,
                 return;         /* done */
         }
 
+	/* draw bop marker */
+	bop = rtw->richtext.bop(rtw->richtext.data, row);
+	switch (bop) {
+		XPoint points[3];
+	case 0:		/* do nothing */
+		break;
+	default:	/* beginning of paragraph */
+#if 1
+		points[0].x = x_base-10;	points[0].y = y_base+4;
+		points[1].x = 7;		points[1].y = height/2-4;
+		points[2].x = -points[1].x;	points[2].y = points[1].y;
+		XSetForeground(display, rtw->richtext.cell_gc,
+			BlackPixelOfScreen(XtScreen(rtw)));
+		XDrawLines(display, cell_win, rtw->richtext.cell_gc,
+			points, 3, CoordModePrevious);
+#else
+		XDrawString(display, cell_win, rtw->richtext.cell_gc,
+			x_base-10, y_base+height, "", 1);
+		rich_char bm;
+		bm.fmt = 0;
+		bm.c = '';
+		rc_draw(cell_win, rtw->richtext.cell_gc,
+			x_base-10, y_base+height, bm, 0);
+#endif
+		break;
+	}
+
         for (i = 0; line[i].c; i++) {
-                int width, vadj;
+                int width;
                 char b[2];
                 b[0] = line[i].c;
                 b[1] = '\0';
-                font_index = (line[i].fmt) & 127;
-                font = get_font(display, font_index);
-                XSetFont(display, rtw->richtext.cell_gc, font);
 
-                switch (line[i].fmt & VADJ_MASK) {
-                case VADJ_TOP:
-                        vadj = -6;
-                        break;
-                case VADJ_BOTTOM:
-                        vadj = 6;
-                        break;
-                default:
-                        vadj = 0;
-                }
-                color_index = ((line[i].fmt) & COLOR_MASK) >> COLOR_SHIFT;
-                color = get_color(display, color_index);
+                color_index = format_table[line[i].fmt].fg;
+                color = get_color(color_index);
                 old_x_base = x_base;
                 XSetForeground(display, rtw->richtext.cell_gc, color);
-                if (inblock(rtw, row, i))
-                        XDrawImageString(display, cell_win,
-				rtw->richtext.cell_gc,
-                                x_base, y_base+height+vadj, b, 1);
-                else
-                        XDrawString(display, cell_win,
-				rtw->richtext.cell_gc,
-                                x_base, y_base+height+vadj, b, 1);
-                if (b[0] == '\t') {     /* hard coded tabs */
-                        x_base += 36;   /* one inch */
-                        x_base -= (x_base % 36);
+                if (inblock(rtw, row, i)) {
+			rc_draw(cell_win, rtw->richtext.cell_gc,
+				x_base, y_base+height, line[i], 1);
                 } else {
-                        width = XTextWidth(font_struct(display, font_index),
-                                        b, 1);
+			rc_draw(cell_win, rtw->richtext.cell_gc,
+				x_base, y_base+height, line[i], 0);
+		}
+                if (b[0] == '\t') {
+			int new_x = x_base - rtw->richtext.left_margin;
+			new_x += rtw->richtext.tab_distance;
+			new_x -= new_x % rtw->richtext.tab_distance;
+			x_base = new_x + rtw->richtext.left_margin;
+                } else {
+			width = rc_width(line[i]);
                         x_base += width;
                 }
-                if (line[i].fmt & ULINE) {
+                if (format_table[line[i].fmt].uline) {
                         XDrawLine(display, cell_win,
 				rtw->richtext.cell_gc,
                                 old_x_base, y_base+height+1,
@@ -588,6 +663,8 @@ static void draw_line(RichtextWidget rtw, Drawable cell_win,
         }
 }
 
+/* ---
+*/
 void rt_pr_line(Widget w, int row)
 {
         Window cell_win = XtWindow(w);
@@ -605,7 +682,6 @@ void rt_pr_line(Widget w, int row)
 		toggle_cursor(rtw);
 }
 
-/* This is the equivalent of the Table widget's draw_table function. */
 void draw_richtext(RichtextWidget rtw, Drawable d)
 {
         int i, y_base;
@@ -618,10 +694,13 @@ void draw_richtext(RichtextWidget rtw, Drawable d)
 			y_base < height; i++) {
                 draw_line(rtw, d, y_base, i, FALSE);
                 y_base += row_height(rtw, i);
-                /* the rest moved to draw_line() */
         }
 }
 
+/* ---
+Draw onto a pixmap.
+*/
+
 Pixmap richtext_pixmap(RichtextWidget rtw)
 {
         unsigned int width, height, depth;
@@ -704,7 +783,7 @@ static Boolean SetValues(Widget current, Widget request, Widget new,
 static void DoLayout(RichtextWidget sw)
 {
         int i;
-        long x, y, top_x, top_y;
+        int x, y, top_x, top_y;
 
         if (sw->composite.num_children) {
                 richtext_global_coords(sw,
@@ -730,7 +809,7 @@ static void DoLayout(RichtextWidget sw)
 static void Resize(Widget w)
 {
         DoLayout((RichtextWidget)w);
-} /* Resize */
+}
 
 /*
  * Geometry Manager
@@ -749,3 +828,17 @@ static void ChangeManaged(Widget w)
     DoLayout((RichtextWidget)w);
 }
 
+int RichtextLookupString(Widget w, XEvent *event, char *buf,
+			 int bufsiz, KeySym *keysym)
+{
+#ifdef HAVE_XCREATEIC
+	Status status;
+	return XmbLookupString(((RichtextWidget)w)->richtext.xic,
+				(XKeyEvent *)event, (char *)buf, bufsiz,
+				keysym, &status);
+#else
+        return XLookupString((XKeyEvent *)event, (char *)buf, bufsiz,
+				keysym, NULL);
+#endif
+}
+
