@@ -25,13 +25,15 @@
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 
-#include "xfonts.h"	/* from common */
+#include "../common/cmalloc.h"
+#include "../common/fonts.h"
+#include "xfonts.h"
 #include "embed.h"
 #include <X11/xpm.h>
 
 #include "TableP.h"
 
-static void plugin_coords(Widget, XtPointer, long *, long *);
+static void plugin_coords(Widget, XtPointer, int *, int *);
 
 #define offset(field) XtOffsetOf(TableRec, table.field)
 static XtResource resources[] = {
@@ -313,7 +315,7 @@ static void Realize(Widget, XtValueMask *, XSetWindowAttributes *);
 static void Destroy(Widget);
 static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal *);
 
-static void plugin_coords(Widget w, XtPointer p, long *x, long *y)
+static void plugin_coords(Widget w, XtPointer p, int *x, int *y)
 {
 	*x = *y = 0;
 }
@@ -386,7 +388,7 @@ static void TableAction(Widget w, XEvent *event, String *params, Cardinal *n)
 	;	/* application should create its own actions */
 }
 
-static int ret_type(Widget w, int row, int col)
+static int return_type(Widget w, int row, int col)
 {
 	TableWidget tw = (TableWidget) w;
 	if (tw->table.type)
@@ -394,7 +396,7 @@ static int ret_type(Widget w, int row, int col)
 	return TABLE_TEXT;
 }
 
-static char *ret_pvalue(Widget w, int row, int col)
+static char *return_pvalue(Widget w, int row, int col)
 {
 	TableWidget tw = (TableWidget) w;
 	if (tw->table.text)
@@ -402,22 +404,28 @@ static char *ret_pvalue(Widget w, int row, int col)
 	else return NULL;
 }
 
-static unsigned long ret_format(Widget w, int row, int col)
+static int return_format(Widget w, int row, int col)
 {
 	TableWidget tw = (TableWidget) w;
+
 	if (tw->table.format)
 		return (*tw->table.format)(tw->table.data, row, col);
-	return HELVETICA | SIZE_10;
+	return 0;
 }
 
-static unsigned long ret_font(Widget w, int row, int col)
+static int return_font(Widget w, int row, int col)
 {
-	return ret_format(w, row, col) & (SIZE_MASK | ITALIC | BOLD | FONT_MASK);
+	return format_table[return_format(w, row, col)].font;
 }
 
-static unsigned long ret_color(Widget w, int row, int col)
+static int return_color(Widget w, int row, int col)
 {
-	return (ret_format(w, row, col) & COLOR_MASK) >> COLOR_SHIFT;
+	return format_table[return_format(w, row, col)].fg;
+}
+
+static int return_bgcolor(Widget w, int row, int col)
+{
+	return format_table[return_format(w, row, col)].bg;
 }
 
 static Dimension cell_width(TableWidget tw, int col)
@@ -441,10 +449,12 @@ static Dimension cell_height(TableWidget tw, int row)
 	return tw->table.default_height;
 }
 
+/* ---
+*/
 void table_global_coords(TableWidget tw,
-		long row, long col, long *x, long *y)
+		int row, int col, int *x, int *y)
 {
-	long i;
+	int i;
 
 	*x = *y = 0;
 
@@ -455,6 +465,12 @@ void table_global_coords(TableWidget tw,
 		*y += cell_height(tw, i);
 }
 
+static int inblock(TableWidget tw, int r, int c)
+{
+	return r >= tw->table.sel_top_row && r <= tw->table.sel_bottom_row
+		&& c >= tw->table.sel_left_col && c <= tw->table.sel_right_col;
+}
+
 
 static void draw_cell(TableWidget tw, Drawable scribble,
 			int row, int col, int x_base, int y_base)
@@ -462,49 +478,50 @@ static void draw_cell(TableWidget tw, Drawable scribble,
 	Widget w = (Widget) tw;
 	char *p;
 	int font_index, color_index;
-	Font font;
 	unsigned long color;
 	int x_pos, y_pos;
 	unsigned int text_height, text_width;
-	unsigned long format;
 	int cw, ch;
+	int ct;
+	int fmt;
+	int siagfmt;
+	rich_char *rcp;
 
 	if (row > tw->table.max_row || col > tw->table.max_col) return;
 
 	cw = cell_width(tw, col);
 	ch = cell_height(tw, row);
 
-	if (ret_type(w, row, col) == TABLE_EMBED) {
-		char *p = ret_pvalue(w, row, col);
+	fmt = return_format(w, row, col);
+
+	ct = return_type(w, row, col);
+	if (ct == TABLE_EMBED) {
+		char *p = return_pvalue(w, row, col);
 		embed_draw(scribble, x_base, y_base, p);
 		return;
 	}
 
-	if (ret_type(w, row, col) == TABLE_PIXMAP) {
+	if (ct == TABLE_PIXMAP) {
 		XtWarning("The Table widget does not yet display pixmaps");
 		return;
 	}
 
-	font_index = ret_font(w, row, col);
-	font = get_font(XtDisplay(w), font_index);
-	XSetFont(XtDisplay(w), tw->table.cell_gc, font);
+	font_index = return_font(w, row, col);
 
-	color_index = ret_color(w, row, col);
-	color = get_color(XtDisplay(w), color_index);
+	color_index = return_color(w, row, col);
+	color = get_color(color_index);
 	XSetForeground(XtDisplay(w), tw->table.cell_gc, color);
 
-	p = ret_pvalue(w, row, col);
+	p = return_pvalue(w, row, col);
 	if (p == NULL) p = "";
+	rcp = rc_makerich(p, fmt);
 
-#if 1	/* this seems to be necessary to keep X from crashing */
 	if (strlen(p) > 1000) p[1000] = '\0';
-#endif
 
-	text_height = font_height(XtDisplay(w), font_index);
-	text_width = XTextWidth(font_struct(XtDisplay(w), font_index),
-				p, strlen(p));
-	format = ret_format(w, row, col);
-	switch (format & HADJ_MASK) {
+	rc_strgeom(rcp, -1, &text_width, &text_height);
+
+	siagfmt = format_table[fmt].siagfmt;
+	switch (siagfmt & HADJ_MASK) {
 	case HADJ_CENTER:
 		x_pos = (cw-text_width) / 2;
 		break;
@@ -515,21 +532,22 @@ static void draw_cell(TableWidget tw, Drawable scribble,
 		x_pos = 5;
 	}
 
-	switch (format & VADJ_MASK) {
+	switch (siagfmt & VADJ_MASK) {
 	case VADJ_BOTTOM:
 		y_pos = ch-5;
 		break;
 	case VADJ_TOP:
-		y_pos = text_height+font_descent(XtDisplay(w), font_index);
+		y_pos = text_height+font_descent(font_index);
 		break;
 	default:	/* VADJ_CENTER */
 		y_pos = (ch+text_height)/2
-			-font_descent(XtDisplay(w), font_index);
+			-font_descent(font_index);
 	}
 
-	XDrawString(XtDisplay(w), scribble, tw->table.cell_gc,
+	rc_strdraw(scribble, tw->table.cell_gc,
 		x_base+x_pos, y_base+y_pos,
-		p, strlen(p));
+		rcp, -1);
+	cfree(rcp);
 	XSetForeground(XtDisplay(w), tw->table.grid_gc, tw->table.black);
 	if (tw->table.grid_lines) {
 		XDrawLine(XtDisplay(w), scribble, tw->table.grid_gc,
@@ -540,16 +558,16 @@ static void draw_cell(TableWidget tw, Drawable scribble,
 			x_base+cw-1, y_base);
 	}
 	/* these lines are from the format */
-	if (format & BORDER_LEFT)
+	if (siagfmt & BORDER_LEFT)
 		XDrawLine(XtDisplay(w), scribble, tw->table.grid_gc,
 			x_base, y_base, x_base, y_base+ch-2);
-	if (format & BORDER_RIGHT)
+	if (siagfmt & BORDER_RIGHT)
 		XDrawLine(XtDisplay(w), scribble, tw->table.grid_gc,
 			x_base+cw-2, y_base, x_base+cw-2, y_base+ch-2);
-	if (format & BORDER_TOP)
+	if (siagfmt & BORDER_TOP)
 		XDrawLine(XtDisplay(w), scribble, tw->table.grid_gc,
 			x_base, y_base, x_base+cw-2, y_base);
-	if (format & BORDER_BOTTOM)
+	if (siagfmt & BORDER_BOTTOM)
 		XDrawLine(XtDisplay(w), scribble, tw->table.grid_gc,
 			x_base, y_base+ch-2, x_base+cw-2, y_base+ch-2);
 	XSetForeground(XtDisplay(w), tw->table.grid_gc,
@@ -562,12 +580,47 @@ static void draw_cell(TableWidget tw, Drawable scribble,
 	}
 }
 
+static void erase_cell(TableWidget tw, Drawable scribble,
+			int row, int col, int x_base, int y_base)
+{
+	Widget w = (Widget) tw;
+	int color_index;
+	unsigned long color;
+	int cw, ch;
+
+	if (row > tw->table.max_row || col > tw->table.max_col) return;
+
+	cw = cell_width(tw, col);
+	ch = cell_height(tw, row);
+
+	if (inblock(tw, row, col)) {
+		XFillRectangle(XtDisplay(w), scribble, tw->table.block_gc,
+			x_base, y_base, cw, ch);
+	} else {
+		color_index = return_bgcolor(w, row, col);
+		color = get_color(color_index);
+		XSetForeground(XtDisplay(w), tw->table.cell_gc, color);
+		XFillRectangle(XtDisplay(w), scribble, tw->table.cell_gc,
+			x_base, y_base, cw, ch);
+	}
+}
+
 static void cell_row(TableWidget tw, Drawable scribble,
 			int width, int y_base, int i)
 {
 	int j, x_base;
 	x_base = 0;
 
+	for (j = 1; (j < tw->table.prot_col) && (x_base < width); j++) {
+		erase_cell(tw, scribble, i, j, x_base, y_base);
+		x_base += cell_width(tw, j);
+	}
+	for (j = tw->table.top_col; x_base < width; j++) {
+		erase_cell(tw, scribble, i, j, x_base, y_base);
+		x_base += cell_width(tw, j);
+	}
+
+	x_base = 0;
 	for (j = 1; (j < tw->table.prot_col) && (x_base < width); j++) {
 		draw_cell(tw, scribble, i, j, x_base, y_base);
 		x_base += cell_width(tw, j);
@@ -619,11 +672,13 @@ static int cell_prev_col(TableWidget w, int col)
         return col-1;
 }
 
+/* ---
+*/
 void table_cell2coords(TableWidget tw,
-			long cell_row, long cell_col,
-			long *cell_x, long *cell_y)
+			int cell_row, int cell_col,
+			int *cell_x, int *cell_y)
 {
-        long i;
+        int i;
         *cell_y = 0;
         for (i = 1; i < tw->table.prot_row; i++)
                 *cell_y += cell_height(tw, i);
@@ -650,11 +705,13 @@ void table_cell2coords(TableWidget tw,
         }
 }
 
+/* ---
+*/
 void table_coords2cell(TableWidget tw,
-			long *cur_row, long *cur_col,
-			long cur_x, long cur_y)
+			int *cur_row, int *cur_col,
+			int cur_x, int cur_y)
 {
-        long prot_x = 0, prot_y = 0, i;
+        int prot_x = 0, prot_y = 0, i;
 
         for (i = 1; i < tw->table.prot_col; i++)
               cur_x -= cell_width(tw, i);
@@ -679,12 +736,12 @@ void table_coords2cell(TableWidget tw,
                 cur_x -= cell_width(tw, *cur_col);
                 (*cur_col) = cell_next_col(tw, *cur_col);
         }
-
 }
 
-/* create, draw and return a pixmap of the stage at time now.
-   Caller must free */
-/* Strategy:
+/* ---
+create, draw and return a pixmap of the stage at time now.
+   Caller must free
+Strategy:
    1. Clear the window, or create a new empty Pixmap
    2. Paint a rectangle for the block, if there is one (coords != 0)
    3. Draw the grid
@@ -693,11 +750,12 @@ void table_coords2cell(TableWidget tw,
    The purpose of using XOR is to make sure the cursor is visible
    on any background. Technicolor is a potential side effect.
 */
+
 Pixmap table_pixmap(Widget w)
 {
 	Pixmap scribble;
 	TableWidget tw = (TableWidget)w;
-	long top_x, top_y, bot_x, bot_y, max_r, max_c, width, height;
+	int top_x, top_y, bot_x, bot_y, max_r, max_c, width, height;
 
 	width = tw->core.width;
 	height = tw->core.height;
@@ -706,8 +764,6 @@ Pixmap table_pixmap(Widget w)
 
 	scribble = XCreatePixmap(XtDisplay(w), XtWindow(w),
 		width, height, tw->core.depth);
-	XFillRectangle(XtDisplay(w), scribble, tw->table.clear_gc,
-		0, 0, width, height);
 	table_cell2coords(tw,
 		tw->table.sel_top_row, tw->table.sel_left_col,
 		&top_x, &top_y);
@@ -716,11 +772,12 @@ Pixmap table_pixmap(Widget w)
 		max_r = tw->table.sel_bottom_row;
 	if (max_c > tw->table.sel_right_col)
 		max_c = tw->table.sel_right_col;
+
 	table_cell2coords(tw, max_r, max_c, &bot_x, &bot_y);
 	if (bot_x >= top_x && bot_x >= 0 && top_x <= width
 			&& bot_y >= top_y && bot_y >= 0 && top_y <= height) {
-		long wi = bot_x-top_x+cell_width(tw, tw->table.sel_right_col);
-		long he = bot_y-top_y+cell_height(tw, tw->table.sel_bottom_row);
+		int wi = bot_x-top_x+cell_width(tw, tw->table.sel_right_col);
+		int he = bot_y-top_y+cell_height(tw, tw->table.sel_bottom_row);
 		if (top_x < 0) {
 			wi += top_x;
 			top_x = 0;
@@ -729,8 +786,6 @@ Pixmap table_pixmap(Widget w)
 			he += top_y;
 			top_y = 0;
 		}
-		XFillRectangle(XtDisplay(w), scribble, tw->table.block_gc,
-			top_x, top_y, wi, he);
 	}
 
 	draw_table(tw, scribble);
@@ -766,12 +821,20 @@ static void Realize(Widget w, XtValueMask *valueMask,
 	tw->table.clear_gc = get_gc(w, tw->table.white, tw->table.black);
 	tw->table.cell_gc = get_gc(w, tw->table.black, tw->table.white);
 	tw->table.grid_gc = get_gc(w, tw->table.black, tw->table.white);
+#if 1	/* this is too much */
+	XSetLineAttributes(XtDisplay(w), tw->table.grid_gc,
+			0, LineSolid, CapButt, JoinMiter);
+#endif
 	tw->table.cursor_gc = get_gc(w, tw->table.black^tw->table.white, 0);
-	tw->table.block_gc = get_gc(w, tw->table.grey, tw->table.white);
 	XSetFunction(XtDisplay(w), tw->table.cursor_gc, GXxor);
-
 	XSetLineAttributes(XtDisplay(w), tw->table.cursor_gc,
 			2, LineSolid, CapButt, JoinMiter);
+#if 1	/* grey block */
+	tw->table.block_gc = get_gc(w, tw->table.grey, tw->table.white);
+#else	/* reverse block */
+	tw->table.block_gc = get_gc(w, tw->table.black^tw->table.white, 0);
+	XSetFunction(XtDisplay(w), tw->table.block_gc, GXxor);
+#endif
 }
 
 static void Destroy(Widget w)
@@ -789,7 +852,7 @@ static void Destroy(Widget w)
 static Boolean move_top(TableWidget tw)
 {
 	Boolean pr_scr_flag = False;
-	long cur_x, cur_y, prot_x = 0, prot_y = 0, i;
+	int cur_x, cur_y, prot_x = 0, prot_y = 0, i;
 	unsigned int width, height;
 
         for (i = 1; i < tw->table.prot_col; i++)
@@ -830,8 +893,8 @@ static Boolean move_top(TableWidget tw)
 
 static void toggle_cursor(TableWidget tw)
 {
-	long row = tw->table.point_row, col = tw->table.point_col;
-	long cur_x, cur_y;
+	int row = tw->table.point_row, col = tw->table.point_col;
+	int cur_x, cur_y;
 	GC gc = tw->table.cursor_gc;
 	unsigned long bg;
 
@@ -876,6 +939,13 @@ static void Redisplay(Widget w, XEvent *event, Region r)
 }
 
 
+/* ---
+A dilemma: if Xt handles the redisplaying, it also takes care of
+   compressing expose events. On the other hand, it clears the window
+   which makes it flicker. We'll redisplay ourselves and try to
+   make it more efficient later.
+*/
+
 static Boolean SetValues(Widget current, Widget request, Widget new,
 		ArgList args, Cardinal *n)
 {
@@ -895,10 +965,6 @@ static Boolean SetValues(Widget current, Widget request, Widget new,
 		do_redisplay = True;
 		newtw->table.redisplay = False;
 	}
-/* A dilemma: if Xt handles the redisplaying, it also takes care of
-   compressing expose events. On the other hand, it clears the window
-   which makes it flicker. We'll redisplay ourselves and try to
-   make it more efficient later. */
 	/* can't let Xt handle this because it flickers */
 	if (do_redisplay) {
 		Redisplay(new, NULL, None);
@@ -917,11 +983,6 @@ static Boolean SetValues(Widget current, Widget request, Widget new,
 		move_top(newtw);
 	do_redisplay = False;
 	Redisplay(new, NULL, None);
-#if 0
-	if (newtw->table.visible_cursor) {
-		toggle_cursor(curtw);
-	}
-#endif
 #endif
 	return do_redisplay;
 }
@@ -933,7 +994,7 @@ static Boolean SetValues(Widget current, Widget request, Widget new,
 static void DoLayout(TableWidget sw)
 {
 	int i;
-	long x, y, top_x, top_y, prot_x, prot_y;
+	int x, y, top_x, top_y, prot_x, prot_y;
 
 	if (sw->composite.num_children) {
 		table_global_coords(sw,
