@@ -1,6 +1,25 @@
-/* plugin.c */
-
 /*
+    Copyright (C) 1996-1999  Ulric Eriksson <ulric@edu.stockholm.se>
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2, or (at your option)
+    any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+    MA 02111-1307, USA.
+*/
+
+/* ---
+plugin.c
+
 The protocol runs over two pipes, connected to the plugin's
 stdin and stdout:
 
@@ -28,6 +47,8 @@ EXEC string	Execute the string, which should be interpretable
 HELP		List available commands
 NOOP		Do nothing
 WIN		Print the window id of window to be captured
+EWIN		Tell the plugin where to send events
+PH		Tell the plugin its handle
 QUIT		Quit
 PRNT		Produce Postscript representation of the plugin
 
@@ -44,8 +65,8 @@ program waits for the reply.
 
 This implementation does *not* handle multi-line replies and will
 likely choke on them. I'm just testing.
-*/
-/*
+
+
 The typical use of the plugin library will be like this:
 
 1. plugin_start: The plugin is loaded into the buffer but not displayed.
@@ -62,7 +83,7 @@ The typical use of the plugin library will be like this:
 4. plugin_hide: The toplevel window is unmapped and the widget
    destroyed.
 5. plugin_stop: plugin is hidden, QUIT is sent and pipes are closed.
-*/
+--- */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -72,18 +93,22 @@ The typical use of the plugin library will be like this:
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <X11/Intrinsic.h>
+#include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
 #include <X11/cursorfont.h>
 #include <X11/xpm.h>
+#include "../common/common.h"
 #include "../common/cmalloc.h"
 #include "plugin.h"
 
 #define REPARENT_LOOPS 50
 #define PLUGIN_MAX 30
 
-/*#define DEBUG*/
+/*#define DEBUG
+*/
+
+#include "../common/traceme.h"
 
 char **plugin_patterns;
 
@@ -107,20 +132,27 @@ static struct {
 static int nhandler = 0;
 
 static Widget topLevel = None;
-static Cursor sane_cursor;
+static Cursor sane_cursor, lr_cursor;
 
-/* write a line to the plugin */
-static int plugin_write(int ph, char *b)
+static Atom plugin_protocol;
+static void (*plugin_execute)(char *) = NULL;
+
+/* ---
+write a line to the plugin
+*/
+
+int plugin_write(int ph, char *b)
 {
 	int retval = write(plugin[ph].pfd[1], b, strlen(b));
-#ifdef DEBUG
-	if (retval >= 0) fprintf(stderr, "plugin_write(%d, %s)\n", ph, b);
-#endif
+
 	return retval;
 }
 
-/* read a line from the plugin, return number of chars read */
-static int plugin_read(int ph, char *b)
+/* ---
+read a line from the plugin, return number of chars read
+*/
+
+int plugin_read(int ph, char *b)
 {
 	fd_set rfds;
 	struct timeval tv;
@@ -128,7 +160,7 @@ static int plugin_read(int ph, char *b)
 
 	FD_ZERO(&rfds);
 	FD_SET(plugin[ph].qfd[0], &rfds);
-	tv.tv_sec = 60;
+	tv.tv_sec = 30;
 	tv.tv_usec = 0;
 	retval = select(plugin[ph].qfd[0]+1, &rfds, NULL, NULL, &tv);
 	if (!retval) return 0;
@@ -144,9 +176,7 @@ static int plugin_read(int ph, char *b)
 	n = read(plugin[ph].qfd[0], b, 1024);
 #endif
 	b[n] = '\0';
-#ifdef DEBUG
-	fprintf(stderr, "plugin_read(%d, %s)\n", ph, b);
-#endif
+
 	return n;
 }
 
@@ -180,11 +210,6 @@ static char **parse_cmd(char *cmd, char *fn, char *name)
 	return argv;
 }
 
-void chomp(char *p)
-{
-	if ((p = strchr(p, '\n'))) *p = '\0';
-}
-
 static int plugin_handler_find(char *ext)
 {
 	int i;
@@ -195,15 +220,16 @@ static int plugin_handler_find(char *ext)
 	return -1;
 }
 
-/*
-	Run a plugin without displaying it.
+/* ---
+Run a plugin without displaying it.
 
-	fn: File name of file to be loaded. The extension is used to
-	    choose from available plugins. All the plumbing for
-	    communicating with the plugin is set up.
+fn: File name of file to be loaded. The extension is used to
+    choose from available plugins. All the plumbing for
+    communicating with the plugin is set up.
 
-	Returns a plugin handle for success of -1 for failure.
+Returns a plugin handle for success of -1 for failure.
 */
+
 int plugin_start(char *fn)
 {
 	int x, y;
@@ -245,11 +271,27 @@ int plugin_start(char *fn)
 		exit(EXIT_FAILURE);
 	} else {				/* parent */
 		char r[1024];
+		int n;
 		close(plugin[ph].pfd[0]);
 		close(plugin[ph].qfd[1]);
+		n = 1;
+		while (n) {
+			n = sleep(n);
+		}
 		if (plugin_read(ph, r) < 6 || r[0] != '2') {
 			return -1;
 		}
+		sprintf(r, "EWIN %lx\n", (unsigned long)XtWindow(topLevel));
+		plugin_write(ph, r);
+		if (plugin_read(ph, r) < 2) {
+			/* it doesn't matter if the plugin returns error */
+			return -1;
+		}
+		sprintf(r, "PH %d\n", ph);
+		plugin_write(ph, r);
+		if (plugin_read(ph, r) < 2) {
+			return -1;
+		}
 		plugin_write(ph, "WIN\n");
 		if (plugin_read(ph, r) < 6 || r[0] != '2') {
 			return -1;
@@ -278,16 +320,43 @@ int plugin_start(char *fn)
 	return ph;
 }
 
-/*
-	Kill a plugin. Close the pipes, close the window, kill the process.
+/* ---
+Resize a plugin.
 
-	ph: Plugin handle.
+ph: Plugin handle
+wi: New width
+he: New height
 
-	Returns 0 for success or non-0 for failure.
+Returns 0 for success or non-0 for failure.
 */
+
+int plugin_size_set(int ph, int width, int height)
+{
+	if (width < 10 || width > 1000 ||
+		height < 10 || height > 1000) return -1;
+	plugin[ph].width = width;
+	plugin[ph].height = height;
+	if (plugin[ph].visible && plugin[ph].core != None) {
+		XtResizeWidget(plugin[ph].core, width+6, height+6, 1);
+		XResizeWindow(XtDisplay(plugin[ph].core),
+			plugin[ph].victim, width, height);
+	}
+	return 0;
+}
+
+
+/* ---
+Kill a plugin. Close the pipes, close the window, kill the process.
+
+ph: Plugin handle.
+
+Returns 0 for success or non-0 for failure.
+*/
+
 int plugin_stop(int ph)
 {
 	char b[1024];
+
 	plugin_hide(ph);
 	plugin_write(ph, "QUIT\n");
 	plugin_read(ph, b);
@@ -298,14 +367,15 @@ int plugin_stop(int ph)
 	return 0;
 }
 
-/*
-	Tell the plugin to load a file.
+/* ---
+Tell the plugin to load a file.
 
-	ph: Plugin handle
-	fn: File name
+ph: Plugin handle
+fn: File name
 
-	Returns 0 for success or non-0 for failure.
+Returns 0 for success or non-0 for failure.
 */
+
 int plugin_load(int ph, char *fn)
 {
 	char cmd[1024];
@@ -316,14 +386,15 @@ int plugin_load(int ph, char *fn)
 	return 0;
 }
 
-/*
-	Tell the plugin to save to a file.
+/* ---
+Tell the plugin to save to a file.
 
-	ph: plugin handle
-	fn: File name
+ph: plugin handle
+fn: File name
 
-	Returns 0 for success or non-0 for failure.
+Returns 0 for success or non-0 for failure.
 */
+
 int plugin_save(int ph, char *fn)
 {
 	char cmd[1024];
@@ -334,14 +405,15 @@ int plugin_save(int ph, char *fn)
 	return 0;
 }
 
-/*
-	Ask the plugin what commands it understands.
+/* ---
+Ask the plugin what commands it understands.
 
-	ph: plugin handle
+ph: plugin handle
 
-	Returns a one-line string with the available commands.
-	Caller must free.
+Returns a one-line string with the available commands.
+Caller must free.
 */
+
 char *plugin_help(int ph)
 {
 	char cmd[1024];
@@ -352,6 +424,10 @@ char *plugin_help(int ph)
 	return cstrdup(cmd);
 }
 
+/* ---
+Find the handle of a plugin running under a widget.
+*/
+
 int plugin_find_by_widget(Widget w)
 {
 	int ph;
@@ -362,6 +438,10 @@ int plugin_find_by_widget(Widget w)
 	return -1;
 }
 
+/* ---
+Find the handle of a plugin running in a window.
+*/
+
 int plugin_find_by_window(Window w)
 {
 	int ph;
@@ -390,14 +470,15 @@ static void handle_destroy(Widget w, XtPointer p, XEvent *event, Boolean *b)
 	}
 }
 
-/*
-	Display the plugin, i.e. map the window and reparent if necessary.
+/* ---
+Display the plugin, i.e. map the window and reparent if necessary.
 
-	win: window pointer
-	ph: plugin handle
+win: window pointer
+ph: plugin handle
 
-	Return 0 for success or non-0 for failure.
+Return 0 for success or non-0 for failure.
 */
+
 int plugin_show(int ph, Widget grid)
 {
         Display *display = XtDisplay(grid);
@@ -412,11 +493,11 @@ int plugin_show(int ph, Widget grid)
 	plugin[ph].visible = 1;
 	/* we don't need to tell Table where to put the widget,
 	   because it will be picked up from plugin_coordinates */
-	plugin[ph].core = XtVaCreateManagedWidget("plugin",
+	plugin[ph].core = XtVaCreateManagedWidget("plugin-parent",
 		compositeWidgetClass, grid,
-		XtNwidth, plugin[ph].width,
-		XtNheight, plugin[ph].height,
-		XtNborderWidth, 0,
+		XtNwidth, plugin[ph].width+6,
+		XtNheight, plugin[ph].height+6,
+		XtNborderWidth, 1,
 		(char *)0);
         to = XtWindow(plugin[ph].core);
         XSync(display, False);
@@ -430,6 +511,7 @@ int plugin_show(int ph, Widget grid)
                 }
 #endif
         }
+	XMoveWindow(display, w, 2, 2);
         XMapWindow(display, w);
         XSync(display, False);
 	XtAddEventHandler(plugin[ph].core,
@@ -447,16 +529,17 @@ int plugin_show(int ph, Widget grid)
 */
 	XDefineCursor(display, w, sane_cursor);
 
-	return 0;
+	return plugin_size_set(ph, plugin[ph].width, plugin[ph].height);
 }
 
-/*
-	Hide the plugin, i.e. unmap the window.
+/* ---
+Hide the plugin, i.e. unmap the window.
 
-	ph: Plugin handle
+ph: Plugin handle
 
-	Returns 0 for success or non-0 for failure.
+Returns 0 for success or non-0 for failure.
 */
+
 int plugin_hide(int ph)
 {
 	int i;
@@ -491,7 +574,13 @@ int plugin_hide(int ph)
 	return 0;
 }
 
-/*
+
+typedef struct {
+        char *chars;
+        XColor xcolor;
+} colors;
+
+/* ---
 Clever little hack to print anything, including plugins that don't know
 how to print themselves. Works the following way:
 1. Create override widget of the right size.
@@ -503,11 +592,6 @@ how to print themselves. Works the following way:
 7. Print the widget as in image plugin.
 */
 
-typedef struct {
-        char *chars;
-        XColor xcolor;
-} colors;
-
 static void prnt(Pixmap pixmap, FILE *fp)
 {
         unsigned int width, height, depth = 8;
@@ -530,6 +614,7 @@ static void prnt(Pixmap pixmap, FILE *fp)
                 return;
         }
         sscanf(data[0], "%d %d %d %d", &width, &height, &ncolors, &cpp);
+
         cm = (colors *)cmalloc(ncolors*sizeof(colors));
 
         for (i = 0; i < ncolors; i++) {
@@ -557,7 +642,7 @@ static void prnt(Pixmap pixmap, FILE *fp)
         fprintf(fp, "%%%%Pages: 1\n");
         fprintf(fp, "%%%%BoundingBox: %d %d %d %d\n", 0, 0, width, height);
         t = time(NULL);
-        fprintf(fp, "%%%%CreationDate: %s\n", ctime(&t));
+        fprintf(fp, "%%%%CreationDate: %s", ctime(&t));
         fprintf(fp, "%%%%EndComments\n");
         fprintf(fp, "%%%%EndProlog\n");
         fprintf(fp, "%%%%Page: 1 1\n\n\n");
@@ -595,18 +680,26 @@ static void prnt(Pixmap pixmap, FILE *fp)
         /* print postscript postblurb */
         fprintf(fp, "%%\n\n");
         fprintf(fp, "grestore\n");
-        fprintf(fp, "%%%%Trailer\n");
+
+#if 0	/* and why not? */
+	fprintf(fp, "%%%%Trailer\n");
+	fprintf(fp, "%%%%EOF\n");
+#endif
 
         for (i = 0; i < ncolors; i++) cfree(cm[i].chars);
         cfree(cm);
         cfree(data);
 }
 
+/* ---
+Print even though the plugin doesn't know how.
+*/
+
 static int print_anyway(int ph, FILE *fp)
 {
 	Widget ow;
-	unsigned long width = plugin[ph].width;
-	unsigned long height = plugin[ph].height;
+	unsigned int width = plugin[ph].width;
+	unsigned int height = plugin[ph].height;
 	int i;
 
 	ow = XtVaCreatePopupShell("plugin_print",
@@ -631,50 +724,82 @@ static int print_anyway(int ph, FILE *fp)
 
 	for (i = 0; i < 1 /*REPARENT_LOOPS*/; i++) {
 		XReparentWindow(XtDisplay(topLevel), plugin[ph].victim,
-			XtWindow(plugin[ph].core), 0, 0);
+			XtWindow(plugin[ph].core), 2, 2);
 		XSync(XtDisplay(topLevel), False);
 	}
 	XtDestroyWidget(ow);
 	return 0;
 }
 
-/*
-	Tell the plugin to print its window into the file.
-
-	ph: Plugin handle
-
-	Returns 0 for success or non-0 for failure, such as an i/o error.
-*/
 #define TRAILER "%%Trailer"
 
-int plugin_print(int ph, FILE *fp)
-{
-	char b[1024];
+/* ---
+Tell the plugin to print its window into the file.
 
-	plugin_write(ph, "PRNT\n");
-	if (!plugin_read(ph, b)) {
-		fprintf(stderr, "Nothing from plugin\n");
-		return 1;
-	}
-	if (strlen(b) < 5 || b[0] != '2') {
-		return print_anyway(ph, fp);
-	}
-	while (plugin_read(ph, b)) {
-		fputs(b, fp);
-		if (!strncmp(b, TRAILER, strlen(TRAILER))) return 0;
-	}
-	return 1;	/* didn't find trailer */
-}
+ph: Plugin handle
+
+Returns 0 for success or non-0 for failure, such as an i/o error.
+*/
 
 /*
-	Register a plugin handler.
-
-	desc: a plaintext description of the plugin
-	ext: magic extension to be handled by this plugin
-	cmd: command to run the plugin
-
-	Returns 0 for success, non-0 for failure.
+This code just doesn't work. There are too many opportunities for
+the plugin to mess up the protocol state by sending data without a
+trailer or by sending extra junk at the end or by sending bogus data.
+I need to make this much more bulletproof. The first step is to better
+show the difference between print data and plugin messages.
+Precede each line with a single space and strip it off here.
+Finish with a line consisting of the single word END.
 */
+
+int plugin_print(int ph, FILE *fp, int x, int y, int trust_plugin)
+{
+	char b[1024];
+	int w, h;
+	int result = 1;
+
+	fprintf(fp, "save\n");
+	fprintf(fp, "%d %d translate\n", x, y);
+	/* figure out how big the clipping rectangle should be */
+	w = plugin[ph].width;
+	h = plugin[ph].height;
+	fprintf(fp, "newpath %d %d moveto %d %d lineto"
+		" %d %d lineto %d %d lineto closepath clip\n",
+		0, 0, 0, h, w, h, w, 0);
+	fprintf(fp, "/showpage {} def\n");	/* disarm this */
+
+	if (trust_plugin) {
+		plugin_write(ph, "PRNT\n");
+		if (!plugin_read(ph, b)) {
+			fprintf(stderr, "Nothing from plugin\n");
+			result = 1;
+		} else if (strlen(b) < 5 || b[0] != '2') {
+			result = print_anyway(ph, fp);
+		} else {
+			while (plugin_read(ph, b)) {
+				if (b[0] != ' ') break;	/* found END */
+				if (strcmp(b, " %%Trailer\n")
+				    && strcmp(b, " %%EOF\n"))
+					fputs(b+1, fp);
+			}
+			result = (b[0] == ' ');	/* didn't find END */
+		}
+	} else {
+		result = print_anyway(ph, fp);
+	}
+	fprintf(fp, "restore\n");
+	return result;
+}
+
+/* ---
+Register a plugin handler.
+
+desc: a plaintext description of the plugin
+ext: magic extension to be handled by this plugin
+cmd: command to run the plugin
+
+Returns 0 for success, non-0 for failure.
+*/
+
 int plugin_register(char *desc, char *ext, char *cmd)
 {
 	char b[1024];
@@ -702,18 +827,211 @@ int plugin_register(char *desc, char *ext, char *cmd)
 	return 0;
 }
 
-void plugin_init(Widget w, void (*handle_exit)(int))
+/* ---
+There are eight cases:
+ - upper left corner
+ - upper border
+ - upper right corner
+ - left border
+ - right border
+ - lower left corner
+ - lower border
+ - lower right corner
+
+Corners resize, borders move. We don't implement moving yet, only
+resizing. Also, we only allow resizing using the lower right corner.
+*/
+
+static void PluginResizeAction(Widget w,
+		XEvent *event, String *params, Cardinal *n)
 {
+	XtAppContext app_context = XtWidgetToApplicationContext(w);
+	int owner_events = True;
+	unsigned int event_mask = ButtonReleaseMask |
+				PointerMotionMask;
+	int pointer_mode = GrabModeAsync;
+	int keyboard_mode = GrabModeAsync;
+	Window confine_to = XtWindow(XtParent(w));
+	static Cursor cursor = None;
+	Time time = CurrentTime;
+	int waiting = True;
+	Dimension wi, he, wi2, he2;
+	int ph = plugin_find_by_widget(w);
+
+	if (ph == -1) return;
+
+	wi = event->xbutton.x;
+	he = event->xbutton.y;
+	XtVaGetValues(w,
+		XtNwidth, &wi2,
+		XtNheight, &he2,
+		(char *)0);
+	if (wi2-wi > 10 || he2-he > 10) return;
+
+	/* snipped from colnum_grab */
+	if (cursor == None)
+		cursor = XCreateFontCursor(XtDisplay(w), XC_bottom_right_corner);
+	XtGrabPointer(w, owner_events, event_mask,
+		pointer_mode, keyboard_mode, confine_to, cursor, time);
+	while (waiting) {
+		XEvent event_return;
+		XtAppNextEvent(app_context, &event_return);
+		if (event_return.type == ButtonRelease) {
+			waiting = False;
+		} else if (event_return.type == MotionNotify) {
+			wi = event_return.xmotion.x;
+			he = event_return.xmotion.y;
+			plugin_size_set(ph, wi, he);
+		} else {
+			XtDispatchEvent(&event_return);
+		}
+	}
+	XtUngrabPointer(w, CurrentTime);
+	pr_scr_flag = TRUE;
+}
+
+static void PluginCursorAction(Widget w,
+	XEvent *event, String *params, Cardinal *n)
+{
+	int x, y;
+	Dimension wi, he;
+
+	x = event->xmotion.x;
+	y = event->xmotion.y;
+	XtVaGetValues(w,
+		XtNwidth, &wi,
+		XtNheight, &he,
+		(char *)0);
+	if (wi-x < 10 && he-y < 10) {
+		XDefineCursor(XtDisplay(w), XtWindow(w), lr_cursor);
+	} else {
+		XDefineCursor(XtDisplay(w), XtWindow(w), None);
+	}
+}
+
+static XtActionsRec actions[] =
+{
+	{"plugin-resize", PluginResizeAction},
+	{"plugin-cursor", PluginCursorAction}
+};
+
+
+/* Handle callbacks from plugins
+
+   The protocol today is:
+
+   1. The application registers a window where events can be sent,
+      and a function that can execute commands in the form of strings.
+   2. An event handler is added to the window which handles ClientMessage
+      events.
+   3. Each started plugin is passed two pieces of information: the
+      window above using the EWIN command, and its plugin handle using
+      the PH command.
+   4. When a plugin wants its parent's attention, it stuffs the following
+      information into a ClientMessage event:
+       - the protocol version, currently always 0
+       - the plugin handle
+       - a ticket: a number used to identify the message.
+   5. The event is sent to the window registered above.
+   6. The event handler receives the event, does a little sanity checking
+      and extracts the information.
+   7. The main application sends the command "WHAT ticket" to the plugin,
+      where ticket is the number from step 4.
+   8. The plugin responds with a string.
+   9. The string is passed to the executor function registered in step 1.
+
+   Now that the communication is established, it is up to the application
+   and plugin to agree on how to continue. For Siag, the commands from the
+   plugin will be handled by the execute function, so the command will
+   simply be a Scheme function call. The function can then do what it
+   is told and the exchange be finished. It may also be that the
+   function comes back to the plugin and asks for more information,
+   or it may feed data to the plugin. Examples:
+
+    - A database plugin will take a SQL query (using the EXEC command).
+      Since the query may take some time to finish, the main application
+      goes about its business. When the query comes back, the plugin calls
+      out to the main application. It uses a command (collect-sql),
+      which goes back to the plugin and collects the data.
+      There is no collect-sql function and no database plugin yet,
+      I'm just making this up.
+    - A plotting plugin uses a timer to periodically call back for
+      updated data. That way real-time updates can be done without
+      any special support in the main application.
+      There is no plotting plugin yet either, although SciPlot looks
+      promising.
+    - A plugin can use the main application's user interface to display
+      error messages.
+
+    The plugin protocol only depends on Unix pipes for the communication.
+    This extension depends on X for its implementation. For user interfaces
+    based on X, such as Xt or Gtk, this is fine. For a plugin that has
+    a text-based interface, or no user interface at all (what would a
+    database plugin need a user interface for), there must be another
+    mechanism.
+*/
+
+static int plugin_message(XEvent *event)
+{
+	return event->type == ClientMessage
+		&& event->xclient.message_type == plugin_protocol
+		&& event->xclient.format == 32;
+}
+
+static void PluginDispatchEvent(Widget w, XtPointer pt, XEvent *e, Boolean *b)
+{
+	int n, ph, ticket;
+	char cmd[1024];
+	if (!plugin_message(e)) {
+#if 0
+		fprintf(stderr, "Not a plugin message\n");
+#endif
+		return;
+	}
+	if (e->xclient.data.l[0] != 0) {
+		fprintf(stderr, "Unknown protocol version\n");
+		return;
+	}
+	ph = e->xclient.data.l[1];
+	ticket = e->xclient.data.l[2];
+	sprintf(cmd, "WHAT %d\n", ticket);
+	n = plugin_write(ph, cmd);
+	if (plugin_read(ph, cmd) >= 2)
+		plugin_execute(cmd);
+}
+/* end of plugin callback code */
+
+
+/* ---
+Initialise the plugin code.
+*/
+
+void plugin_init(Widget w, void (*handle_exit)(int), void (*exec_cmd)(char *))
+{
+	Display *dpy;
+	XtAppContext app_context = XtWidgetToApplicationContext(w);
 	topLevel = w;
 	handle_plugin_exit = handle_exit;
 	sane_cursor = XCreateFontCursor(XtDisplay(w), XC_top_left_arrow);
+	lr_cursor = XCreateFontCursor(XtDisplay(w), XC_bottom_right_corner);
+	XtAppAddActions(app_context, actions, XtNumber(actions));
+
+	/* experimental code to get attention from plugins */
+	dpy = XtDisplay(w);
+	plugin_protocol = XInternAtom(dpy, "_PLUGIN_PROTOCOL", False);
+	plugin_execute = exec_cmd;
+	XtAddEventHandler(w, NoEventMask, True,
+			PluginDispatchEvent, NULL);
 }
 
-int plugin_size_get(int ph, unsigned long *width, unsigned long *height)
+/* ---
+Returns the size of a plugin
+*/
+
+int plugin_size_get(int ph, int *width, int *height)
 {
 	if (ph < 0 || ph >= nplugin) return -1;
 	*width = plugin[ph].width;
 	*height = plugin[ph].height;
 	return 0;
 }
-
