@@ -1,3 +1,22 @@
+/*
+   Pathetic Writer
+   Copyright (C) 1997-1999  Ulric Eriksson <ulric@edu.stockholm.se>
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+   MA 02111-1307, USA.
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -22,6 +41,8 @@
 #include "../xcommon/xcommon.h"
 #include "xpw.h"
 
+/* ---
+*/
 int ask_for_str_comp(char *prompt, char *buffr, int (*comp)(char *))
 {
 	return dialog_input(topLevel, prompt, buffr, comp);
@@ -32,73 +53,80 @@ static int nocomp(char *b)
 	return TRUE;
 }
 
+/* ---
+*/
 int ask_for_str(char *prompt, char *buffr)
 {
 	return ask_for_str_comp(prompt, buffr, nocomp);
 }
 
+/* ---
+*/
 int select_file(char *path, char *name, char *patterns[], char *fmt)
 {
 	char extra[1024];
-	sprintf(extra, "Home=%s:Examples=%s/examples/pw",
-		getenv("HOME"), siagdocs);
+	sprintf(extra, "Home=%s:Examples=%s/pw/examples",
+		getenv("HOME"), docdir);
 	return fsel_input(topLevel, path, name, patterns, fmt, extra);
 }
 
+/* ---
+*/
 int alert_box(char *text, char *buttons[], int nbuttons)
 {
 	return alertbox(topLevel, text, buttons, nbuttons);
 }
 
+/* ---
+*/
 int select_from_list(char *text, char *choices[], int nchoices)
 {
 	return listsel(topLevel, text, choices, nchoices);
 }
 
+/* ---
+*/
 void error_box(char *message)
 {
 	errorbox(topLevel, message);
 }
 
-/* the interpretation of this is changed so that the return value
-        is the width of the col:th character in row. All places
-        where cell_width is called must be changed to char_width.
+/* ---
+The interpretation of this is changed so that the return value
+is the width of the col:th character in row. All places
+where cell_width is called must be changed to char_width.
 
-        Precondition: col *must* be a legal position within the line */
-int char_width(buffer *buf, int row, int col)
+Precondition: col *must* be a legal position within the line
+*/
+
+int char_width(buffer *buf, int s, int row, int col)
 {
-        rich_char *line = buf->text[row].p;
-        int i, l, w;
-        char *p;
-        XFontStruct *font_info;
+        rich_char *line = buf->sht[s].text[row].p;
+        int w;
 
         if (line == NULL) return 0;     /* do not dump core */
 
-        i = 0;                          /* accumulated index */
-        p = (char *)rc_makeplain(line); /* pointer to character */
-        l = rc_strlen(line);            /* line length */
-        if (col > l) return 0;          /* lose */
-        font_info = font_struct(display, line[col].fmt & 127);
-        w = XTextWidth(font_info, p+col, 1);
-        cfree(p);
+	w = rc_width(line[col]);
         return w;
 }
 
-/* width of the line up to but not including col */
-int line_width(buffer *buf, int row, int col)
+/* ---
+width of the line up to but not including col
+*/
+
+int line_width(buffer *buf, int s, int row, int col)
 {
         rich_char *line;
         char *p;
         int i, l;
         unsigned int width;
-        XFontStruct *font_info;
 
-        if (row > buf->used_lines) return 0;
-        line = buf->text[row].p;
+        if (row > buf->sht[s].used_lines) return 0;
+        line = buf->sht[s].text[row].p;
         if (line == NULL) return 0;     /* do not dump core */
         p = (char *)rc_makeplain(line);
 
-        if (buf->text[row].sty == STY_EMBED) {
+        if (buf->sht[s].text[row].sty == STY_EMBED) {
                 if (col == 0) return 0;
                 embed_size(p, &width, NULL);
                 return width;
@@ -109,11 +137,10 @@ int line_width(buffer *buf, int row, int col)
         width = 0;      /* accumulated width */
         for (i = 0; i < col && line[i].c; i++) {
                 if (line[i].c == '\t') {
-                        width += 36;
-                        width -= (width % 36);
+                        width += buf->tab_distance;
+                        width -= (width % tab_distance);
                 } else {
-                        font_info = font_struct(display, line[i].fmt & 127);
-                        width += XTextWidth(font_info, p+i, 1);
+			width += rc_width(line[i]);
                 }
         }
         cfree(p);
@@ -182,26 +209,31 @@ static XtActionsRec actions[] =
 #endif
 };
 
-/* move point inside visible area */
+/* ---
+move point inside visible area
+*/
+
 static void fix_point(int width, int height)
 {
         int cur_x, cur_y;
         buffer *buf = w_list->buf;
+	int s = w_list->sht;
+	sheet *st = buf->sht;
 
         /* do the easy stuff first */
-        if (w_list->point_pos.row < w_list->top.row)
-                w_list->point_pos.row = w_list->top.row;
+        if (st[s].point_pos.row < st[s].top.row)
+                st[s].point_pos.row = st[s].top.row;
 
-        get_char_coords(w_list, w_list->top.row, w_list->top.col,
-                        w_list->point_pos.row, w_list->point_pos.col,
+        get_char_coords(w_list, st[s].top.row, st[s].top.col,
+                        st[s].point_pos.row, st[s].point_pos.col,
                         &cur_x, &cur_y);
         while (cur_y < 0) {
-                w_list->point_pos.row++;
-                cur_y += cell_height(buf, w_list->point_pos.row);
+                st[s].point_pos.row++;
+                cur_y += cell_height(buf, s, st[s].point_pos.row);
         }
-        while (cur_y > height - cell_height(buf, w_list->point_pos.row)) {
-                w_list->point_pos.row--;
-                cur_y -= cell_height(buf, w_list->point_pos.row);
+        while (cur_y > height - cell_height(buf, s, st[s].point_pos.row)) {
+                st[s].point_pos.row--;
+                cur_y -= cell_height(buf, s, st[s].point_pos.row);
         }
 }
 
@@ -211,18 +243,20 @@ static LISP scroll_up(void)
         int x, y, new_y, new_row;
         unsigned int width, height, border_width, depth;
         buffer *buf = w_list->buf;
+	int s = w_list->sht;
+	sheet *st = buf->sht;
 
         XGetGeometry(display, char_win, &root, &x, &y, &width, &height,
                      &border_width, &depth);
-        new_y = -height + cell_height(buf, w_list->point_pos.row);
-        new_row = w_list->top.row;
+        new_y = -height + cell_height(buf, s, st[s].point_pos.row);
+        new_row = st[s].top.row;
         while (new_y < 0) {
-                new_y += cell_height(buf, w_list->point_pos.row);
+                new_y += cell_height(buf, s, st[s].point_pos.row);
                 new_row--;
         }
         if (new_row < 1)
                 new_row = 1;
-        w_list->top.row = new_row;
+        st[s].top.row = new_row;
         fix_point(width, height);
         pr_scr_flag = TRUE;
         return NIL;
@@ -234,18 +268,20 @@ static LISP scroll_char_up(void)
         int x, y, new_y, new_row;
         unsigned int width, height, border_width, depth;
         buffer *buf = w_list->buf;
+	int s = w_list->sht;
+	sheet *st = buf->sht;
 
         XGetGeometry(display, char_win, &root, &x, &y, &width, &height,
                      &border_width, &depth);
-        new_y = -cell_height(buf, w_list->point_pos.row);
-        new_row = w_list->top.row;
+        new_y = -cell_height(buf, s, st[s].point_pos.row);
+        new_row = st[s].top.row;
         while (new_y < 0) {
-                new_y += cell_height(buf, w_list->point_pos.row);
+                new_y += cell_height(buf, s, st[s].point_pos.row);
                 new_row--;
         }
         if (new_row < 1)
                 new_row = 1;
-        w_list->top.row = new_row;
+        st[s].top.row = new_row;
         fix_point(width, height);
         pr_scr_flag = TRUE;
         return NIL;
@@ -257,18 +293,20 @@ static LISP scroll_down(void)
         int x, y, new_y, new_row;
         unsigned int width, height, border_width, depth;
         buffer *buf = w_list->buf;
+	int s = w_list->sht;
+	sheet *st = buf->sht;
 
         XGetGeometry(display, char_win, &root, &x, &y, &width, &height,
                      &border_width, &depth);
-        new_y = height - cell_height(buf, w_list->point_pos.row);
-        new_row = w_list->top.row;
+        new_y = height - cell_height(buf, s, st[s].point_pos.row);
+        new_row = st[s].top.row;
         while (new_y > 0) {
-                new_y -= cell_height(buf, w_list->point_pos.row);
+                new_y -= cell_height(buf, s, st[s].point_pos.row);
                 new_row++;
         }
         if (new_row > max_lines)
                 new_row = max_lines;
-        w_list->top.row = new_row;
+        st[s].top.row = new_row;
         fix_point(width, height);
         pr_scr_flag = TRUE;
         return NIL;
@@ -280,18 +318,20 @@ static LISP scroll_char_down(void)
         int x, y, new_y, new_row;
         unsigned int width, height, border_width, depth;
         buffer *buf = w_list->buf;
+	int s = w_list->sht;
+	sheet *st = buf->sht;
 
         XGetGeometry(display, char_win, &root, &x, &y, &width, &height,
                      &border_width, &depth);
-        new_y = cell_height(buf, w_list->point_pos.row);
-        new_row = w_list->top.row;
+        new_y = cell_height(buf, s, st[s].point_pos.row);
+        new_row = st[s].top.row;
         while (new_y > 0) {
-                new_y -= cell_height(buf, w_list->point_pos.row);
+                new_y -= cell_height(buf, s, st[s].point_pos.row);
                 new_row++;
         }
         if (new_row > max_lines)
                 new_row = 1;
-        w_list->top.row = new_row;
+        st[s].top.row = new_row;
         fix_point(width, height);
         pr_scr_flag = TRUE;
         return NIL;
@@ -301,28 +341,30 @@ static LISP set_block(void)
 {
         position p = get_point(w_list);
         position m = get_mark(w_list);
+	int s = w_list->sht;
+	sheet *st = w_list->buf->sht;
 
         if (p.row < m.row) {
-                w_list->blku.row = p.row;
-                w_list->blkl.row = m.row;
+                st[s].blkl.row = p.row;
+                st[s].blkl.row = m.row;
         } else {
-                w_list->blku.row = m.row;
-                w_list->blkl.row = p.row;
+                st[s].blku.row = m.row;
+                st[s].blkl.row = p.row;
         }
         if (p.col < m.col) {
-                w_list->blku.col = p.col;
-                w_list->blkl.col = m.col;
+                st[s].blku.col = p.col;
+                st[s].blkl.col = m.col;
         } else {
-                w_list->blku.col = m.col;
-                w_list->blkl.col = p.col;
+                st[s].blku.col = m.col;
+                st[s].blkl.col = p.col;
         }
 
         if (XtOwnSelection(w_list->ui->grid, XA_PRIMARY,
 			CurrentTime, convert_proc,
                 lose_ownership_proc, transfer_done_proc) == False) {
                         XtWarning("Siag: failed to become selection owner\n");
-                        w_list->blku.row = w_list->blku.col = -1;
-                        w_list->blkl.row = w_list->blkl.col = -1;
+                        st[s].blku.row = st[s].blku.col = -1;
+                        st[s].blkl.row = st[s].blkl.col = -1;
         }
 
         pr_scr_flag = TRUE;
@@ -340,9 +382,11 @@ static LISP lembed_object(void)
         char file[256], *tag;
         unsigned int width, height;
         buffer *buf = w_list->buf;
-        int row = w_list->point_pos.row;
+	int s = w_list->sht;
+	sheet *st = buf->sht;
+        int row = st[s].point_pos.row;
 
-        if (ret_style(buf, row) == STY_EMBED) {
+        if (ret_style(buf, s, row) == STY_EMBED) {
                 llpr("Can't overwrite embedded object");
                 return NIL;
         }
@@ -354,10 +398,10 @@ static LISP lembed_object(void)
         if (tag == NULL) return NIL;
 
         embed_size(tag, &width, &height);
-        buf->text[row].height = height;
-        ins_text(buf, make_position(row, 0), (unsigned char *)tag,
-                        HELVETICA | SIZE_12);
-        buf->text[row].sty = STY_EMBED;
+        buf->sht[s].text[row].height = height;
+        ins_text(buf, s, make_position(row, 0), (unsigned char *)tag,
+                        0);
+        buf->sht[s].text[row].sty = STY_EMBED;
         w_list->buf->change = TRUE;
         pr_scr_flag = TRUE;
         return NIL;
@@ -366,10 +410,12 @@ static LISP lembed_object(void)
 static LISP lembed_remove(void)
 {
         buffer *buf = w_list->buf;
-        int row = w_list->point_pos.row;
-        if (buf->text[row].sty == STY_EMBED) {
-                w_list->point_pos.col = 0;
-                del_char(buf, row, 0);
+	int s = w_list->sht;
+	sheet *st = buf->sht;
+        int row = st[s].point_pos.row;
+        if (buf->sht[s].text[row].sty == STY_EMBED) {
+                st[s].point_pos.col = 0;
+                del_char(buf, s, row, 0);
         }
         w_list->buf->change = TRUE;
         pr_scr_flag = TRUE;
@@ -379,9 +425,11 @@ static LISP lembed_remove(void)
 static LISP lembed_open(void)
 {
         buffer *buf = w_list->buf;
-        int row = w_list->point_pos.row;
-        if (buf->text[row].sty == STY_EMBED) {
-                char *p = (char *)rc_makeplain(buf->text[row].p);
+	int s = w_list->sht;
+	sheet *st = buf->sht;
+        int row = st[s].point_pos.row;
+        if (buf->sht[s].text[row].sty == STY_EMBED) {
+                char *p = (char *)rc_makeplain(buf->sht[s].text[row].p);
                 embed_open(p);
                 embed_load(p);
                 cfree(p);
@@ -408,170 +456,8 @@ static LISP lembed_save(void)
         return NIL;
 }
 
-/* cool, all this was copied from Siag with cut and paste */
-static LISP lplugin_register(LISP desc, LISP ext, LISP cmd)
-{
-        plugin_register(get_c_string(desc),
-                        get_c_string(ext),
-                        get_c_string(cmd));
-        return NIL;
-}
-
-#define PLUGIN_LINK 1
-#define PLUGIN_COPY 2
-
-static void insert_plugin(int mode, char *plugin_name)
-{
-        static char path[1024], name[1024];
-        char fn[1024];
-        char pn[1024];
-        char cmd[1024];
-        char fmt[80];
-        char *p;
-        buffer *buf = w_list->buf;
-        int n;
-        plugin_t plugin;
-
-        /* ask for file name */
-        if (path[0] == '\0') getcwd(path, 1024);
-        name[0] = fn[0] = '\0';
-
-        if (plugin_name) {
-                char *q = strrchr(plugin_name, '/');
-                if (q) strcpy(name, q+1);
-                else strcpy(name, plugin_name);
-                strcpy(fn, plugin_name);
-        } else {
-                if (!select_file(path, name, plugin_patterns, fmt)) return;
-                sprintf(fn, "%s/%s", path, name);
-        }
-
-        plugin_unique_name(name, pn);
-
-        plugin.row = w_list->point_pos.row;
-        plugin.col = w_list->point_pos.col;
-        plugin.displayed = 0;
-
-        if (mode == PLUGIN_COPY) {
-                plugin.name = cstrdup(pn);
-                /* copy the file */
-                p = plugin_basedir(buf, NULL);
-                sprintf(pn, "%s/%s", p, plugin.name);
-                sprintf(cmd, "(mkdir %s;cp %s %s)2>/dev/null", p, fn, pn);
-                system(cmd);
-        } else {
-                strcpy(pn, fn);
-                plugin.name = cstrdup(pn);
-        }
-
-        /* start the plugin */
-        plugin.ph = plugin_start(pn);
-        if (plugin.ph == -1) return;
-
-        /* prepare the buffer */
-        n = buf->nplugin++;
-        buf->plugin = (plugin_t *)crealloc(buf->plugin,
-                buf->nplugin*sizeof(plugin_t));
-        buf->plugin[n] = plugin;
-        buf->change = TRUE;
-        pr_scr_flag = TRUE;
-}
-
-static LISP lplugin_import(LISP pn)
-{
-        if (NULLP(pn)) insert_plugin(PLUGIN_COPY, NULL);
-        else insert_plugin(PLUGIN_COPY, get_c_string(pn));
-        return NIL;
-}
-
-/* broken because it still takes the name relative to basedir */
-static LISP lplugin_link(LISP pn)
-{
-        if (NULLP(pn)) insert_plugin(PLUGIN_LINK, NULL);
-        else insert_plugin(PLUGIN_LINK, get_c_string(pn));
-        return NIL;
-}
-
-/* return buffer's plug handler index or -1 for error */
-static int select_plugin(buffer *b)
-{
-        char *plugin[20];
-        int i;
-
-        if (b->nplugin == 0) return -1;
-        for (i = 0; i < 20 && i < b->nplugin; i++)
-                plugin[i] = b->plugin[i].name;
-        i = select_from_list("Select Plugin:", plugin, i);
-        return i;
-}
-
-static LISP lplugin_export(void)
-{
-        int n;
-        char fn[1024], pn[1024], cmd[1024], path[1024], fmt[80];
-        buffer *buf = w_list->buf;
-
-        /* pick plugin from list */
-        n = select_plugin(buf);
-        if (n == -1) return NIL;
-
-        /* get description of the handler */
-        /* ask for file name */
-        getcwd(path, 1024);
-        strcpy(fn, buf->plugin[n].name);
-        if (!select_file(path, fn, NULL, fmt)) return NIL;
-
-        /* save the file */
-        sprintf(pn, "%s/%s", plugin_basedir(buf, NULL), buf->plugin[n].name);
-        sprintf(cmd, "cp %s %s/%s", pn, path, fn);
-        system(cmd);
-        return NIL;
-}
-
-static LISP lplugin_create(void)
-{
-        /* get list of registered handlers */
-        /* ask for file name */
-        /* start the plugin */
-        return NIL;
-}
-
-static LISP lplugin_delete(void)
-{
-        int n;
-        buffer *buf = w_list->buf;
-
-        /* pick plugin from list */
-        n = select_plugin(buf);
-        if (n == -1) return NIL;
-
-        /* remove plugin */
-        plugin_stop(buf->plugin[n].ph);
-        cfree(buf->plugin[n].name);
-        buf->nplugin--;
-        for (; n < buf->nplugin; n++)
-                buf->plugin[n] = buf->plugin[n+1];
-        buf->change = pr_scr_flag = TRUE;
-        return NIL;
-}
-
-static LISP lplugin_move(void)
-{
-        int n;
-        buffer *buf = w_list->buf;
-
-        /* pick plugin from list */
-        n = select_plugin(buf);
-        if (n == -1) return NIL;
-
-	alloc_line(buf, buf->plugin[n].row);
-	buf->text[buf->plugin[n].row].height = styles[STY_DEFAULT].height;
-        buf->plugin[n].row = w_list->point_pos.row;
-        buf->plugin[n].col = w_list->point_pos.col;
-        buf->change = pr_scr_flag = TRUE;
-        return NIL;
-}
-
+/* ---
+*/
 void interp_startup(void)
 {
         XtAppContext app_context = XtWidgetToApplicationContext(topLevel);
@@ -587,47 +473,40 @@ void interp_startup(void)
 	init_subr_0("embed-remove", lembed_remove);
 	init_subr_0("embed-open", lembed_open);
 	init_subr_0("embed-save", lembed_save);
-        init_subr_3("plugin-register", lplugin_register);
-        init_subr_1("plugin-import", lplugin_import);
-	init_subr_1("plugin-link", lplugin_link);
-        init_subr_0("plugin-export", lplugin_export);
-        init_subr_0("plugin-create", lplugin_create);
-        init_subr_0("plugin-delete", lplugin_delete);
-        init_subr_0("plugin-move", lplugin_move);
 }
 
-/* Postscript font handling. Metrics are taken from X */
-char *ps_fontname(int index)
-{
-	return fonts[index].psfont;
-}
+/* ---
+Postscript font handling. Metrics are taken from X
+*/
 
 int ps_text_width(int index, char *s)
 {
-	return XTextWidth(font_struct(XtDisplay(topLevel), index), s, strlen(s));
+	return font_width(index, s);
 }
 
+/* ---
+*/
 int ps_font_descent(int index)
 {
-	return font_descent(XtDisplay(topLevel), index);
+	return font_descent(index);
 }
 
+/* ---
+*/
 int ps_font_height(long font)
 {
-	return font_height(XtDisplay(topLevel), font);
+	return font_height(font);
 }
 
+/* ---
+*/
 int ps_embed_print(FILE *fp, char *tag, int x_base, int y_base)
 {
 	return embed_print(fp, tag, x_base, y_base);
 }
 
-int ps_font_size(int index)
-{
-	return font_size[index];
-}
 
-/*
+/* ---
 	A dialog for spelling checkers. This should probably be in xcommon.
 
 	The dialog has the following components:
@@ -644,7 +523,7 @@ int ps_font_size(int index)
 	A command button with the word "Help"
 
 	The dialog terminates when any of the buttons other than Help
-	is pressed. Help pops up Siaghelp with the file pw/docs/spell.html.
+	is pressed. Help pops up Siaghelp with the file pw/spell.html.
 
 	The function accepts these parameters:
 
@@ -659,7 +538,7 @@ int ps_font_size(int index)
 
 	Lots of code swiped from forminput.c, which is where this
 	really belongs.
-*/
+--- */
 
 spell_state state;
 
@@ -705,8 +584,7 @@ static void spell_cb(Widget w, XtPointer client_data, XtPointer call_data)
 	if (data == SPELL_HELP) {
 		char b[256];
 
-		sprintf(b, "file://localhost%s/pw/docs/spell.html",
-			siagdocs);
+		sprintf(b, "file://localhost%s/pw/spell.html", docdir);
 		if (!fork()) {
 			execlp(siaghelp, "Siaghelp", b, (char *)0);
 			exit(0);
@@ -761,6 +639,8 @@ static void new_line(void)
 
 extern void center(Widget, Widget);
 
+/* ---
+*/
 spell_state spell_select(char *original, char *replacement)
 {
 	Widget spell_shell, spell_form, spell_label1, spell_text1,
@@ -818,29 +698,3 @@ spell_state spell_select(char *original, char *replacement)
 	return state;
 }
 
-/* plugin functions declared in user_interface.h */
-extern int pw_plugin_start(char *fn)
-{
-        return plugin_start(fn);
-}
-
-extern int pw_plugin_stop(int ph)
-{
-        return plugin_stop(ph);
-}
-
-extern int pw_plugin_save(int ph, char *fn)
-{
-        return plugin_save(ph, fn);
-}
-
-extern int pw_plugin_size_get(int ph, unsigned long *w, unsigned long *h)
-{
-	return plugin_size_get(ph, w, h);
-}
-
-extern int pw_plugin_print(int ph, FILE *fp)
-{
-	return plugin_print(ph, fp);
-}
-
